from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class LuminanceUnits(Enum):
        """
            LuminanceUnits enumeration
        """
        
        CandelaPerSquareMeter = 'CandelaPerSquareMeter'
        """
            
        """
        
        CandelaPerSquareFoot = 'CandelaPerSquareFoot'
        """
            
        """
        
        CandelaPerSquareInch = 'CandelaPerSquareInch'
        """
            
        """
        
        Nit = 'Nit'
        """
            
        """
        
        NanocandelaPerSquareMeter = 'NanocandelaPerSquareMeter'
        """
            
        """
        
        MicrocandelaPerSquareMeter = 'MicrocandelaPerSquareMeter'
        """
            
        """
        
        MillicandelaPerSquareMeter = 'MillicandelaPerSquareMeter'
        """
            
        """
        
        CenticandelaPerSquareMeter = 'CenticandelaPerSquareMeter'
        """
            
        """
        
        DecicandelaPerSquareMeter = 'DecicandelaPerSquareMeter'
        """
            
        """
        
        KilocandelaPerSquareMeter = 'KilocandelaPerSquareMeter'
        """
            
        """
        

class LuminanceDto:
    """
    A DTO representation of a Luminance

    Attributes:
        value (float): The value of the Luminance.
        unit (LuminanceUnits): The specific unit that the Luminance value is representing.
    """

    def __init__(self, value: float, unit: LuminanceUnits):
        """
        Create a new DTO representation of a Luminance

        Parameters:
            value (float): The value of the Luminance.
            unit (LuminanceUnits): The specific unit that the Luminance value is representing.
        """
        self.value: float = value
        """
        The value of the Luminance
        """
        self.unit: LuminanceUnits = unit
        """
        The specific unit that the Luminance value is representing
        """

    def to_json(self):
        """
        Get a Luminance DTO JSON object representing the current unit.

        :return: JSON object represents Luminance DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "CandelaPerSquareMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Luminance DTO from a json representation.

        :param data: The Luminance DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "CandelaPerSquareMeter"}
        :return: A new instance of LuminanceDto.
        :rtype: LuminanceDto
        """
        return LuminanceDto(value=data["value"], unit=LuminanceUnits(data["unit"]))


class Luminance(AbstractMeasure):
    """
    None

    Args:
        value (float): The value.
        from_unit (LuminanceUnits): The Luminance unit to create from, The default unit is CandelaPerSquareMeter
    """
    def __init__(self, value: float, from_unit: LuminanceUnits = LuminanceUnits.CandelaPerSquareMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__candelas_per_square_meter = None
        
        self.__candelas_per_square_foot = None
        
        self.__candelas_per_square_inch = None
        
        self.__nits = None
        
        self.__nanocandelas_per_square_meter = None
        
        self.__microcandelas_per_square_meter = None
        
        self.__millicandelas_per_square_meter = None
        
        self.__centicandelas_per_square_meter = None
        
        self.__decicandelas_per_square_meter = None
        
        self.__kilocandelas_per_square_meter = None
        

    def convert(self, unit: LuminanceUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: LuminanceUnits = LuminanceUnits.CandelaPerSquareMeter) -> LuminanceDto:
        """
        Get a new instance of Luminance DTO representing the current unit.

        :param hold_in_unit: The specific Luminance unit to store the Luminance value in the DTO representation.
        :type hold_in_unit: LuminanceUnits
        :return: A new instance of LuminanceDto.
        :rtype: LuminanceDto
        """
        return LuminanceDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: LuminanceUnits = LuminanceUnits.CandelaPerSquareMeter):
        """
        Get a Luminance DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Luminance unit to store the Luminance value in the DTO representation.
        :type hold_in_unit: LuminanceUnits
        :return: JSON object represents Luminance DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "CandelaPerSquareMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(luminance_dto: LuminanceDto):
        """
        Obtain a new instance of Luminance from a DTO unit object.

        :param luminance_dto: The Luminance DTO representation.
        :type luminance_dto: LuminanceDto
        :return: A new instance of Luminance.
        :rtype: Luminance
        """
        return Luminance(luminance_dto.value, luminance_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Luminance from a DTO unit json representation.

        :param data: The Luminance DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "CandelaPerSquareMeter"}
        :return: A new instance of Luminance.
        :rtype: Luminance
        """
        return Luminance.from_dto(LuminanceDto.from_json(data))

    def __convert_from_base(self, from_unit: LuminanceUnits) -> float:
        value = self._value
        
        if from_unit == LuminanceUnits.CandelaPerSquareMeter:
            return (value)
        
        if from_unit == LuminanceUnits.CandelaPerSquareFoot:
            return (value/ 1.07639e1)
        
        if from_unit == LuminanceUnits.CandelaPerSquareInch:
            return (value/ 1.5500031e3)
        
        if from_unit == LuminanceUnits.Nit:
            return (value)
        
        if from_unit == LuminanceUnits.NanocandelaPerSquareMeter:
            return ((value) / 1e-09)
        
        if from_unit == LuminanceUnits.MicrocandelaPerSquareMeter:
            return ((value) / 1e-06)
        
        if from_unit == LuminanceUnits.MillicandelaPerSquareMeter:
            return ((value) / 0.001)
        
        if from_unit == LuminanceUnits.CenticandelaPerSquareMeter:
            return ((value) / 0.01)
        
        if from_unit == LuminanceUnits.DecicandelaPerSquareMeter:
            return ((value) / 0.1)
        
        if from_unit == LuminanceUnits.KilocandelaPerSquareMeter:
            return ((value) / 1000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: LuminanceUnits) -> float:
        
        if to_unit == LuminanceUnits.CandelaPerSquareMeter:
            return (value)
        
        if to_unit == LuminanceUnits.CandelaPerSquareFoot:
            return (value* 1.07639e1)
        
        if to_unit == LuminanceUnits.CandelaPerSquareInch:
            return (value* 1.5500031e3)
        
        if to_unit == LuminanceUnits.Nit:
            return (value)
        
        if to_unit == LuminanceUnits.NanocandelaPerSquareMeter:
            return ((value) * 1e-09)
        
        if to_unit == LuminanceUnits.MicrocandelaPerSquareMeter:
            return ((value) * 1e-06)
        
        if to_unit == LuminanceUnits.MillicandelaPerSquareMeter:
            return ((value) * 0.001)
        
        if to_unit == LuminanceUnits.CenticandelaPerSquareMeter:
            return ((value) * 0.01)
        
        if to_unit == LuminanceUnits.DecicandelaPerSquareMeter:
            return ((value) * 0.1)
        
        if to_unit == LuminanceUnits.KilocandelaPerSquareMeter:
            return ((value) * 1000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_candelas_per_square_meter(candelas_per_square_meter: float):
        """
        Create a new instance of Luminance from a value in candelas_per_square_meter.

        

        :param meters: The Luminance value in candelas_per_square_meter.
        :type candelas_per_square_meter: float
        :return: A new instance of Luminance.
        :rtype: Luminance
        """
        return Luminance(candelas_per_square_meter, LuminanceUnits.CandelaPerSquareMeter)

    
    @staticmethod
    def from_candelas_per_square_foot(candelas_per_square_foot: float):
        """
        Create a new instance of Luminance from a value in candelas_per_square_foot.

        

        :param meters: The Luminance value in candelas_per_square_foot.
        :type candelas_per_square_foot: float
        :return: A new instance of Luminance.
        :rtype: Luminance
        """
        return Luminance(candelas_per_square_foot, LuminanceUnits.CandelaPerSquareFoot)

    
    @staticmethod
    def from_candelas_per_square_inch(candelas_per_square_inch: float):
        """
        Create a new instance of Luminance from a value in candelas_per_square_inch.

        

        :param meters: The Luminance value in candelas_per_square_inch.
        :type candelas_per_square_inch: float
        :return: A new instance of Luminance.
        :rtype: Luminance
        """
        return Luminance(candelas_per_square_inch, LuminanceUnits.CandelaPerSquareInch)

    
    @staticmethod
    def from_nits(nits: float):
        """
        Create a new instance of Luminance from a value in nits.

        

        :param meters: The Luminance value in nits.
        :type nits: float
        :return: A new instance of Luminance.
        :rtype: Luminance
        """
        return Luminance(nits, LuminanceUnits.Nit)

    
    @staticmethod
    def from_nanocandelas_per_square_meter(nanocandelas_per_square_meter: float):
        """
        Create a new instance of Luminance from a value in nanocandelas_per_square_meter.

        

        :param meters: The Luminance value in nanocandelas_per_square_meter.
        :type nanocandelas_per_square_meter: float
        :return: A new instance of Luminance.
        :rtype: Luminance
        """
        return Luminance(nanocandelas_per_square_meter, LuminanceUnits.NanocandelaPerSquareMeter)

    
    @staticmethod
    def from_microcandelas_per_square_meter(microcandelas_per_square_meter: float):
        """
        Create a new instance of Luminance from a value in microcandelas_per_square_meter.

        

        :param meters: The Luminance value in microcandelas_per_square_meter.
        :type microcandelas_per_square_meter: float
        :return: A new instance of Luminance.
        :rtype: Luminance
        """
        return Luminance(microcandelas_per_square_meter, LuminanceUnits.MicrocandelaPerSquareMeter)

    
    @staticmethod
    def from_millicandelas_per_square_meter(millicandelas_per_square_meter: float):
        """
        Create a new instance of Luminance from a value in millicandelas_per_square_meter.

        

        :param meters: The Luminance value in millicandelas_per_square_meter.
        :type millicandelas_per_square_meter: float
        :return: A new instance of Luminance.
        :rtype: Luminance
        """
        return Luminance(millicandelas_per_square_meter, LuminanceUnits.MillicandelaPerSquareMeter)

    
    @staticmethod
    def from_centicandelas_per_square_meter(centicandelas_per_square_meter: float):
        """
        Create a new instance of Luminance from a value in centicandelas_per_square_meter.

        

        :param meters: The Luminance value in centicandelas_per_square_meter.
        :type centicandelas_per_square_meter: float
        :return: A new instance of Luminance.
        :rtype: Luminance
        """
        return Luminance(centicandelas_per_square_meter, LuminanceUnits.CenticandelaPerSquareMeter)

    
    @staticmethod
    def from_decicandelas_per_square_meter(decicandelas_per_square_meter: float):
        """
        Create a new instance of Luminance from a value in decicandelas_per_square_meter.

        

        :param meters: The Luminance value in decicandelas_per_square_meter.
        :type decicandelas_per_square_meter: float
        :return: A new instance of Luminance.
        :rtype: Luminance
        """
        return Luminance(decicandelas_per_square_meter, LuminanceUnits.DecicandelaPerSquareMeter)

    
    @staticmethod
    def from_kilocandelas_per_square_meter(kilocandelas_per_square_meter: float):
        """
        Create a new instance of Luminance from a value in kilocandelas_per_square_meter.

        

        :param meters: The Luminance value in kilocandelas_per_square_meter.
        :type kilocandelas_per_square_meter: float
        :return: A new instance of Luminance.
        :rtype: Luminance
        """
        return Luminance(kilocandelas_per_square_meter, LuminanceUnits.KilocandelaPerSquareMeter)

    
    @property
    def candelas_per_square_meter(self) -> float:
        """
        
        """
        if self.__candelas_per_square_meter != None:
            return self.__candelas_per_square_meter
        self.__candelas_per_square_meter = self.__convert_from_base(LuminanceUnits.CandelaPerSquareMeter)
        return self.__candelas_per_square_meter

    
    @property
    def candelas_per_square_foot(self) -> float:
        """
        
        """
        if self.__candelas_per_square_foot != None:
            return self.__candelas_per_square_foot
        self.__candelas_per_square_foot = self.__convert_from_base(LuminanceUnits.CandelaPerSquareFoot)
        return self.__candelas_per_square_foot

    
    @property
    def candelas_per_square_inch(self) -> float:
        """
        
        """
        if self.__candelas_per_square_inch != None:
            return self.__candelas_per_square_inch
        self.__candelas_per_square_inch = self.__convert_from_base(LuminanceUnits.CandelaPerSquareInch)
        return self.__candelas_per_square_inch

    
    @property
    def nits(self) -> float:
        """
        
        """
        if self.__nits != None:
            return self.__nits
        self.__nits = self.__convert_from_base(LuminanceUnits.Nit)
        return self.__nits

    
    @property
    def nanocandelas_per_square_meter(self) -> float:
        """
        
        """
        if self.__nanocandelas_per_square_meter != None:
            return self.__nanocandelas_per_square_meter
        self.__nanocandelas_per_square_meter = self.__convert_from_base(LuminanceUnits.NanocandelaPerSquareMeter)
        return self.__nanocandelas_per_square_meter

    
    @property
    def microcandelas_per_square_meter(self) -> float:
        """
        
        """
        if self.__microcandelas_per_square_meter != None:
            return self.__microcandelas_per_square_meter
        self.__microcandelas_per_square_meter = self.__convert_from_base(LuminanceUnits.MicrocandelaPerSquareMeter)
        return self.__microcandelas_per_square_meter

    
    @property
    def millicandelas_per_square_meter(self) -> float:
        """
        
        """
        if self.__millicandelas_per_square_meter != None:
            return self.__millicandelas_per_square_meter
        self.__millicandelas_per_square_meter = self.__convert_from_base(LuminanceUnits.MillicandelaPerSquareMeter)
        return self.__millicandelas_per_square_meter

    
    @property
    def centicandelas_per_square_meter(self) -> float:
        """
        
        """
        if self.__centicandelas_per_square_meter != None:
            return self.__centicandelas_per_square_meter
        self.__centicandelas_per_square_meter = self.__convert_from_base(LuminanceUnits.CenticandelaPerSquareMeter)
        return self.__centicandelas_per_square_meter

    
    @property
    def decicandelas_per_square_meter(self) -> float:
        """
        
        """
        if self.__decicandelas_per_square_meter != None:
            return self.__decicandelas_per_square_meter
        self.__decicandelas_per_square_meter = self.__convert_from_base(LuminanceUnits.DecicandelaPerSquareMeter)
        return self.__decicandelas_per_square_meter

    
    @property
    def kilocandelas_per_square_meter(self) -> float:
        """
        
        """
        if self.__kilocandelas_per_square_meter != None:
            return self.__kilocandelas_per_square_meter
        self.__kilocandelas_per_square_meter = self.__convert_from_base(LuminanceUnits.KilocandelaPerSquareMeter)
        return self.__kilocandelas_per_square_meter

    
    def to_string(self, unit: LuminanceUnits = LuminanceUnits.CandelaPerSquareMeter, fractional_digits: int = None) -> str:
        """
        Format the Luminance to a string.
        
        Note: the default format for Luminance is CandelaPerSquareMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Luminance. Default is 'CandelaPerSquareMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == LuminanceUnits.CandelaPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.candelas_per_square_meter, fractional_digits)} Cd/m²"""
        
        if unit == LuminanceUnits.CandelaPerSquareFoot:
            return f"""{super()._truncate_fraction_digits(self.candelas_per_square_foot, fractional_digits)} Cd/ft²"""
        
        if unit == LuminanceUnits.CandelaPerSquareInch:
            return f"""{super()._truncate_fraction_digits(self.candelas_per_square_inch, fractional_digits)} Cd/in²"""
        
        if unit == LuminanceUnits.Nit:
            return f"""{super()._truncate_fraction_digits(self.nits, fractional_digits)} nt"""
        
        if unit == LuminanceUnits.NanocandelaPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.nanocandelas_per_square_meter, fractional_digits)} nCd/m²"""
        
        if unit == LuminanceUnits.MicrocandelaPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.microcandelas_per_square_meter, fractional_digits)} μCd/m²"""
        
        if unit == LuminanceUnits.MillicandelaPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.millicandelas_per_square_meter, fractional_digits)} mCd/m²"""
        
        if unit == LuminanceUnits.CenticandelaPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.centicandelas_per_square_meter, fractional_digits)} cCd/m²"""
        
        if unit == LuminanceUnits.DecicandelaPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.decicandelas_per_square_meter, fractional_digits)} dCd/m²"""
        
        if unit == LuminanceUnits.KilocandelaPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.kilocandelas_per_square_meter, fractional_digits)} kCd/m²"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: LuminanceUnits = LuminanceUnits.CandelaPerSquareMeter) -> str:
        """
        Get Luminance unit abbreviation.
        Note! the default abbreviation for Luminance is CandelaPerSquareMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == LuminanceUnits.CandelaPerSquareMeter:
            return """Cd/m²"""
        
        if unit_abbreviation == LuminanceUnits.CandelaPerSquareFoot:
            return """Cd/ft²"""
        
        if unit_abbreviation == LuminanceUnits.CandelaPerSquareInch:
            return """Cd/in²"""
        
        if unit_abbreviation == LuminanceUnits.Nit:
            return """nt"""
        
        if unit_abbreviation == LuminanceUnits.NanocandelaPerSquareMeter:
            return """nCd/m²"""
        
        if unit_abbreviation == LuminanceUnits.MicrocandelaPerSquareMeter:
            return """μCd/m²"""
        
        if unit_abbreviation == LuminanceUnits.MillicandelaPerSquareMeter:
            return """mCd/m²"""
        
        if unit_abbreviation == LuminanceUnits.CenticandelaPerSquareMeter:
            return """cCd/m²"""
        
        if unit_abbreviation == LuminanceUnits.DecicandelaPerSquareMeter:
            return """dCd/m²"""
        
        if unit_abbreviation == LuminanceUnits.KilocandelaPerSquareMeter:
            return """kCd/m²"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class VolumePerLengthUnits(Enum):
        """
            VolumePerLengthUnits enumeration
        """
        
        CubicMeterPerMeter = 'CubicMeterPerMeter'
        """
            
        """
        
        LiterPerMeter = 'LiterPerMeter'
        """
            
        """
        
        LiterPerKilometer = 'LiterPerKilometer'
        """
            
        """
        
        LiterPerMillimeter = 'LiterPerMillimeter'
        """
            
        """
        
        OilBarrelPerFoot = 'OilBarrelPerFoot'
        """
            
        """
        
        CubicYardPerFoot = 'CubicYardPerFoot'
        """
            
        """
        
        CubicYardPerUsSurveyFoot = 'CubicYardPerUsSurveyFoot'
        """
            
        """
        
        UsGallonPerMile = 'UsGallonPerMile'
        """
            
        """
        
        ImperialGallonPerMile = 'ImperialGallonPerMile'
        """
            
        """
        

class VolumePerLengthDto:
    """
    A DTO representation of a VolumePerLength

    Attributes:
        value (float): The value of the VolumePerLength.
        unit (VolumePerLengthUnits): The specific unit that the VolumePerLength value is representing.
    """

    def __init__(self, value: float, unit: VolumePerLengthUnits):
        """
        Create a new DTO representation of a VolumePerLength

        Parameters:
            value (float): The value of the VolumePerLength.
            unit (VolumePerLengthUnits): The specific unit that the VolumePerLength value is representing.
        """
        self.value: float = value
        """
        The value of the VolumePerLength
        """
        self.unit: VolumePerLengthUnits = unit
        """
        The specific unit that the VolumePerLength value is representing
        """

    def to_json(self):
        """
        Get a VolumePerLength DTO JSON object representing the current unit.

        :return: JSON object represents VolumePerLength DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "CubicMeterPerMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of VolumePerLength DTO from a json representation.

        :param data: The VolumePerLength DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "CubicMeterPerMeter"}
        :return: A new instance of VolumePerLengthDto.
        :rtype: VolumePerLengthDto
        """
        return VolumePerLengthDto(value=data["value"], unit=VolumePerLengthUnits(data["unit"]))


class VolumePerLength(AbstractMeasure):
    """
    Volume, typically of fluid, that a container can hold within a unit of length.

    Args:
        value (float): The value.
        from_unit (VolumePerLengthUnits): The VolumePerLength unit to create from, The default unit is CubicMeterPerMeter
    """
    def __init__(self, value: float, from_unit: VolumePerLengthUnits = VolumePerLengthUnits.CubicMeterPerMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__cubic_meters_per_meter = None
        
        self.__liters_per_meter = None
        
        self.__liters_per_kilometer = None
        
        self.__liters_per_millimeter = None
        
        self.__oil_barrels_per_foot = None
        
        self.__cubic_yards_per_foot = None
        
        self.__cubic_yards_per_us_survey_foot = None
        
        self.__us_gallons_per_mile = None
        
        self.__imperial_gallons_per_mile = None
        

    def convert(self, unit: VolumePerLengthUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: VolumePerLengthUnits = VolumePerLengthUnits.CubicMeterPerMeter) -> VolumePerLengthDto:
        """
        Get a new instance of VolumePerLength DTO representing the current unit.

        :param hold_in_unit: The specific VolumePerLength unit to store the VolumePerLength value in the DTO representation.
        :type hold_in_unit: VolumePerLengthUnits
        :return: A new instance of VolumePerLengthDto.
        :rtype: VolumePerLengthDto
        """
        return VolumePerLengthDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: VolumePerLengthUnits = VolumePerLengthUnits.CubicMeterPerMeter):
        """
        Get a VolumePerLength DTO JSON object representing the current unit.

        :param hold_in_unit: The specific VolumePerLength unit to store the VolumePerLength value in the DTO representation.
        :type hold_in_unit: VolumePerLengthUnits
        :return: JSON object represents VolumePerLength DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "CubicMeterPerMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(volume_per_length_dto: VolumePerLengthDto):
        """
        Obtain a new instance of VolumePerLength from a DTO unit object.

        :param volume_per_length_dto: The VolumePerLength DTO representation.
        :type volume_per_length_dto: VolumePerLengthDto
        :return: A new instance of VolumePerLength.
        :rtype: VolumePerLength
        """
        return VolumePerLength(volume_per_length_dto.value, volume_per_length_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of VolumePerLength from a DTO unit json representation.

        :param data: The VolumePerLength DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "CubicMeterPerMeter"}
        :return: A new instance of VolumePerLength.
        :rtype: VolumePerLength
        """
        return VolumePerLength.from_dto(VolumePerLengthDto.from_json(data))

    def __convert_from_base(self, from_unit: VolumePerLengthUnits) -> float:
        value = self._value
        
        if from_unit == VolumePerLengthUnits.CubicMeterPerMeter:
            return (value)
        
        if from_unit == VolumePerLengthUnits.LiterPerMeter:
            return (value * 1000)
        
        if from_unit == VolumePerLengthUnits.LiterPerKilometer:
            return (value * 1e6)
        
        if from_unit == VolumePerLengthUnits.LiterPerMillimeter:
            return (value)
        
        if from_unit == VolumePerLengthUnits.OilBarrelPerFoot:
            return (value * 1.91713408)
        
        if from_unit == VolumePerLengthUnits.CubicYardPerFoot:
            return (value / 2.50838208)
        
        if from_unit == VolumePerLengthUnits.CubicYardPerUsSurveyFoot:
            return (value / 2.50837706323584)
        
        if from_unit == VolumePerLengthUnits.UsGallonPerMile:
            return (value * (1000 * 1609.344 / 3.785411784))
        
        if from_unit == VolumePerLengthUnits.ImperialGallonPerMile:
            return (value * (1000 * 1609.344 / 4.54609))
        
        return None


    def __convert_to_base(self, value: float, to_unit: VolumePerLengthUnits) -> float:
        
        if to_unit == VolumePerLengthUnits.CubicMeterPerMeter:
            return (value)
        
        if to_unit == VolumePerLengthUnits.LiterPerMeter:
            return (value / 1000)
        
        if to_unit == VolumePerLengthUnits.LiterPerKilometer:
            return (value / 1e6)
        
        if to_unit == VolumePerLengthUnits.LiterPerMillimeter:
            return (value)
        
        if to_unit == VolumePerLengthUnits.OilBarrelPerFoot:
            return (value / 1.91713408)
        
        if to_unit == VolumePerLengthUnits.CubicYardPerFoot:
            return (value * 2.50838208)
        
        if to_unit == VolumePerLengthUnits.CubicYardPerUsSurveyFoot:
            return (value * 2.50837706323584)
        
        if to_unit == VolumePerLengthUnits.UsGallonPerMile:
            return (value / (1000 * 1609.344 / 3.785411784))
        
        if to_unit == VolumePerLengthUnits.ImperialGallonPerMile:
            return (value / (1000 * 1609.344 / 4.54609))
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_cubic_meters_per_meter(cubic_meters_per_meter: float):
        """
        Create a new instance of VolumePerLength from a value in cubic_meters_per_meter.

        

        :param meters: The VolumePerLength value in cubic_meters_per_meter.
        :type cubic_meters_per_meter: float
        :return: A new instance of VolumePerLength.
        :rtype: VolumePerLength
        """
        return VolumePerLength(cubic_meters_per_meter, VolumePerLengthUnits.CubicMeterPerMeter)

    
    @staticmethod
    def from_liters_per_meter(liters_per_meter: float):
        """
        Create a new instance of VolumePerLength from a value in liters_per_meter.

        

        :param meters: The VolumePerLength value in liters_per_meter.
        :type liters_per_meter: float
        :return: A new instance of VolumePerLength.
        :rtype: VolumePerLength
        """
        return VolumePerLength(liters_per_meter, VolumePerLengthUnits.LiterPerMeter)

    
    @staticmethod
    def from_liters_per_kilometer(liters_per_kilometer: float):
        """
        Create a new instance of VolumePerLength from a value in liters_per_kilometer.

        

        :param meters: The VolumePerLength value in liters_per_kilometer.
        :type liters_per_kilometer: float
        :return: A new instance of VolumePerLength.
        :rtype: VolumePerLength
        """
        return VolumePerLength(liters_per_kilometer, VolumePerLengthUnits.LiterPerKilometer)

    
    @staticmethod
    def from_liters_per_millimeter(liters_per_millimeter: float):
        """
        Create a new instance of VolumePerLength from a value in liters_per_millimeter.

        

        :param meters: The VolumePerLength value in liters_per_millimeter.
        :type liters_per_millimeter: float
        :return: A new instance of VolumePerLength.
        :rtype: VolumePerLength
        """
        return VolumePerLength(liters_per_millimeter, VolumePerLengthUnits.LiterPerMillimeter)

    
    @staticmethod
    def from_oil_barrels_per_foot(oil_barrels_per_foot: float):
        """
        Create a new instance of VolumePerLength from a value in oil_barrels_per_foot.

        

        :param meters: The VolumePerLength value in oil_barrels_per_foot.
        :type oil_barrels_per_foot: float
        :return: A new instance of VolumePerLength.
        :rtype: VolumePerLength
        """
        return VolumePerLength(oil_barrels_per_foot, VolumePerLengthUnits.OilBarrelPerFoot)

    
    @staticmethod
    def from_cubic_yards_per_foot(cubic_yards_per_foot: float):
        """
        Create a new instance of VolumePerLength from a value in cubic_yards_per_foot.

        

        :param meters: The VolumePerLength value in cubic_yards_per_foot.
        :type cubic_yards_per_foot: float
        :return: A new instance of VolumePerLength.
        :rtype: VolumePerLength
        """
        return VolumePerLength(cubic_yards_per_foot, VolumePerLengthUnits.CubicYardPerFoot)

    
    @staticmethod
    def from_cubic_yards_per_us_survey_foot(cubic_yards_per_us_survey_foot: float):
        """
        Create a new instance of VolumePerLength from a value in cubic_yards_per_us_survey_foot.

        

        :param meters: The VolumePerLength value in cubic_yards_per_us_survey_foot.
        :type cubic_yards_per_us_survey_foot: float
        :return: A new instance of VolumePerLength.
        :rtype: VolumePerLength
        """
        return VolumePerLength(cubic_yards_per_us_survey_foot, VolumePerLengthUnits.CubicYardPerUsSurveyFoot)

    
    @staticmethod
    def from_us_gallons_per_mile(us_gallons_per_mile: float):
        """
        Create a new instance of VolumePerLength from a value in us_gallons_per_mile.

        

        :param meters: The VolumePerLength value in us_gallons_per_mile.
        :type us_gallons_per_mile: float
        :return: A new instance of VolumePerLength.
        :rtype: VolumePerLength
        """
        return VolumePerLength(us_gallons_per_mile, VolumePerLengthUnits.UsGallonPerMile)

    
    @staticmethod
    def from_imperial_gallons_per_mile(imperial_gallons_per_mile: float):
        """
        Create a new instance of VolumePerLength from a value in imperial_gallons_per_mile.

        

        :param meters: The VolumePerLength value in imperial_gallons_per_mile.
        :type imperial_gallons_per_mile: float
        :return: A new instance of VolumePerLength.
        :rtype: VolumePerLength
        """
        return VolumePerLength(imperial_gallons_per_mile, VolumePerLengthUnits.ImperialGallonPerMile)

    
    @property
    def cubic_meters_per_meter(self) -> float:
        """
        
        """
        if self.__cubic_meters_per_meter != None:
            return self.__cubic_meters_per_meter
        self.__cubic_meters_per_meter = self.__convert_from_base(VolumePerLengthUnits.CubicMeterPerMeter)
        return self.__cubic_meters_per_meter

    
    @property
    def liters_per_meter(self) -> float:
        """
        
        """
        if self.__liters_per_meter != None:
            return self.__liters_per_meter
        self.__liters_per_meter = self.__convert_from_base(VolumePerLengthUnits.LiterPerMeter)
        return self.__liters_per_meter

    
    @property
    def liters_per_kilometer(self) -> float:
        """
        
        """
        if self.__liters_per_kilometer != None:
            return self.__liters_per_kilometer
        self.__liters_per_kilometer = self.__convert_from_base(VolumePerLengthUnits.LiterPerKilometer)
        return self.__liters_per_kilometer

    
    @property
    def liters_per_millimeter(self) -> float:
        """
        
        """
        if self.__liters_per_millimeter != None:
            return self.__liters_per_millimeter
        self.__liters_per_millimeter = self.__convert_from_base(VolumePerLengthUnits.LiterPerMillimeter)
        return self.__liters_per_millimeter

    
    @property
    def oil_barrels_per_foot(self) -> float:
        """
        
        """
        if self.__oil_barrels_per_foot != None:
            return self.__oil_barrels_per_foot
        self.__oil_barrels_per_foot = self.__convert_from_base(VolumePerLengthUnits.OilBarrelPerFoot)
        return self.__oil_barrels_per_foot

    
    @property
    def cubic_yards_per_foot(self) -> float:
        """
        
        """
        if self.__cubic_yards_per_foot != None:
            return self.__cubic_yards_per_foot
        self.__cubic_yards_per_foot = self.__convert_from_base(VolumePerLengthUnits.CubicYardPerFoot)
        return self.__cubic_yards_per_foot

    
    @property
    def cubic_yards_per_us_survey_foot(self) -> float:
        """
        
        """
        if self.__cubic_yards_per_us_survey_foot != None:
            return self.__cubic_yards_per_us_survey_foot
        self.__cubic_yards_per_us_survey_foot = self.__convert_from_base(VolumePerLengthUnits.CubicYardPerUsSurveyFoot)
        return self.__cubic_yards_per_us_survey_foot

    
    @property
    def us_gallons_per_mile(self) -> float:
        """
        
        """
        if self.__us_gallons_per_mile != None:
            return self.__us_gallons_per_mile
        self.__us_gallons_per_mile = self.__convert_from_base(VolumePerLengthUnits.UsGallonPerMile)
        return self.__us_gallons_per_mile

    
    @property
    def imperial_gallons_per_mile(self) -> float:
        """
        
        """
        if self.__imperial_gallons_per_mile != None:
            return self.__imperial_gallons_per_mile
        self.__imperial_gallons_per_mile = self.__convert_from_base(VolumePerLengthUnits.ImperialGallonPerMile)
        return self.__imperial_gallons_per_mile

    
    def to_string(self, unit: VolumePerLengthUnits = VolumePerLengthUnits.CubicMeterPerMeter, fractional_digits: int = None) -> str:
        """
        Format the VolumePerLength to a string.
        
        Note: the default format for VolumePerLength is CubicMeterPerMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the VolumePerLength. Default is 'CubicMeterPerMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == VolumePerLengthUnits.CubicMeterPerMeter:
            return f"""{super()._truncate_fraction_digits(self.cubic_meters_per_meter, fractional_digits)} m³/m"""
        
        if unit == VolumePerLengthUnits.LiterPerMeter:
            return f"""{super()._truncate_fraction_digits(self.liters_per_meter, fractional_digits)} l/m"""
        
        if unit == VolumePerLengthUnits.LiterPerKilometer:
            return f"""{super()._truncate_fraction_digits(self.liters_per_kilometer, fractional_digits)} l/km"""
        
        if unit == VolumePerLengthUnits.LiterPerMillimeter:
            return f"""{super()._truncate_fraction_digits(self.liters_per_millimeter, fractional_digits)} l/mm"""
        
        if unit == VolumePerLengthUnits.OilBarrelPerFoot:
            return f"""{super()._truncate_fraction_digits(self.oil_barrels_per_foot, fractional_digits)} bbl/ft"""
        
        if unit == VolumePerLengthUnits.CubicYardPerFoot:
            return f"""{super()._truncate_fraction_digits(self.cubic_yards_per_foot, fractional_digits)} yd³/ft"""
        
        if unit == VolumePerLengthUnits.CubicYardPerUsSurveyFoot:
            return f"""{super()._truncate_fraction_digits(self.cubic_yards_per_us_survey_foot, fractional_digits)} yd³/ftUS"""
        
        if unit == VolumePerLengthUnits.UsGallonPerMile:
            return f"""{super()._truncate_fraction_digits(self.us_gallons_per_mile, fractional_digits)} gal (U.S.)/mi"""
        
        if unit == VolumePerLengthUnits.ImperialGallonPerMile:
            return f"""{super()._truncate_fraction_digits(self.imperial_gallons_per_mile, fractional_digits)} gal (imp.)/mi"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: VolumePerLengthUnits = VolumePerLengthUnits.CubicMeterPerMeter) -> str:
        """
        Get VolumePerLength unit abbreviation.
        Note! the default abbreviation for VolumePerLength is CubicMeterPerMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == VolumePerLengthUnits.CubicMeterPerMeter:
            return """m³/m"""
        
        if unit_abbreviation == VolumePerLengthUnits.LiterPerMeter:
            return """l/m"""
        
        if unit_abbreviation == VolumePerLengthUnits.LiterPerKilometer:
            return """l/km"""
        
        if unit_abbreviation == VolumePerLengthUnits.LiterPerMillimeter:
            return """l/mm"""
        
        if unit_abbreviation == VolumePerLengthUnits.OilBarrelPerFoot:
            return """bbl/ft"""
        
        if unit_abbreviation == VolumePerLengthUnits.CubicYardPerFoot:
            return """yd³/ft"""
        
        if unit_abbreviation == VolumePerLengthUnits.CubicYardPerUsSurveyFoot:
            return """yd³/ftUS"""
        
        if unit_abbreviation == VolumePerLengthUnits.UsGallonPerMile:
            return """gal (U.S.)/mi"""
        
        if unit_abbreviation == VolumePerLengthUnits.ImperialGallonPerMile:
            return """gal (imp.)/mi"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class BitRateUnits(Enum):
        """
            BitRateUnits enumeration
        """
        
        BitPerSecond = 'BitPerSecond'
        """
            
        """
        
        BytePerSecond = 'BytePerSecond'
        """
            
        """
        
        KilobitPerSecond = 'KilobitPerSecond'
        """
            
        """
        
        MegabitPerSecond = 'MegabitPerSecond'
        """
            
        """
        
        GigabitPerSecond = 'GigabitPerSecond'
        """
            
        """
        
        TerabitPerSecond = 'TerabitPerSecond'
        """
            
        """
        
        PetabitPerSecond = 'PetabitPerSecond'
        """
            
        """
        
        ExabitPerSecond = 'ExabitPerSecond'
        """
            
        """
        
        KibibitPerSecond = 'KibibitPerSecond'
        """
            
        """
        
        MebibitPerSecond = 'MebibitPerSecond'
        """
            
        """
        
        GibibitPerSecond = 'GibibitPerSecond'
        """
            
        """
        
        TebibitPerSecond = 'TebibitPerSecond'
        """
            
        """
        
        PebibitPerSecond = 'PebibitPerSecond'
        """
            
        """
        
        ExbibitPerSecond = 'ExbibitPerSecond'
        """
            
        """
        
        KilobytePerSecond = 'KilobytePerSecond'
        """
            
        """
        
        MegabytePerSecond = 'MegabytePerSecond'
        """
            
        """
        
        GigabytePerSecond = 'GigabytePerSecond'
        """
            
        """
        
        TerabytePerSecond = 'TerabytePerSecond'
        """
            
        """
        
        PetabytePerSecond = 'PetabytePerSecond'
        """
            
        """
        
        ExabytePerSecond = 'ExabytePerSecond'
        """
            
        """
        
        KibibytePerSecond = 'KibibytePerSecond'
        """
            
        """
        
        MebibytePerSecond = 'MebibytePerSecond'
        """
            
        """
        
        GibibytePerSecond = 'GibibytePerSecond'
        """
            
        """
        
        TebibytePerSecond = 'TebibytePerSecond'
        """
            
        """
        
        PebibytePerSecond = 'PebibytePerSecond'
        """
            
        """
        
        ExbibytePerSecond = 'ExbibytePerSecond'
        """
            
        """
        

class BitRateDto:
    """
    A DTO representation of a BitRate

    Attributes:
        value (float): The value of the BitRate.
        unit (BitRateUnits): The specific unit that the BitRate value is representing.
    """

    def __init__(self, value: float, unit: BitRateUnits):
        """
        Create a new DTO representation of a BitRate

        Parameters:
            value (float): The value of the BitRate.
            unit (BitRateUnits): The specific unit that the BitRate value is representing.
        """
        self.value: float = value
        """
        The value of the BitRate
        """
        self.unit: BitRateUnits = unit
        """
        The specific unit that the BitRate value is representing
        """

    def to_json(self):
        """
        Get a BitRate DTO JSON object representing the current unit.

        :return: JSON object represents BitRate DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "BitPerSecond"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of BitRate DTO from a json representation.

        :param data: The BitRate DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "BitPerSecond"}
        :return: A new instance of BitRateDto.
        :rtype: BitRateDto
        """
        return BitRateDto(value=data["value"], unit=BitRateUnits(data["unit"]))


class BitRate(AbstractMeasure):
    """
    In telecommunications and computing, bit rate is the number of bits that are conveyed or processed per unit of time.

    Args:
        value (float): The value.
        from_unit (BitRateUnits): The BitRate unit to create from, The default unit is BitPerSecond
    """
    def __init__(self, value: float, from_unit: BitRateUnits = BitRateUnits.BitPerSecond):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__bits_per_second = None
        
        self.__bytes_per_second = None
        
        self.__kilobits_per_second = None
        
        self.__megabits_per_second = None
        
        self.__gigabits_per_second = None
        
        self.__terabits_per_second = None
        
        self.__petabits_per_second = None
        
        self.__exabits_per_second = None
        
        self.__kibibits_per_second = None
        
        self.__mebibits_per_second = None
        
        self.__gibibits_per_second = None
        
        self.__tebibits_per_second = None
        
        self.__pebibits_per_second = None
        
        self.__exbibits_per_second = None
        
        self.__kilobytes_per_second = None
        
        self.__megabytes_per_second = None
        
        self.__gigabytes_per_second = None
        
        self.__terabytes_per_second = None
        
        self.__petabytes_per_second = None
        
        self.__exabytes_per_second = None
        
        self.__kibibytes_per_second = None
        
        self.__mebibytes_per_second = None
        
        self.__gibibytes_per_second = None
        
        self.__tebibytes_per_second = None
        
        self.__pebibytes_per_second = None
        
        self.__exbibytes_per_second = None
        

    def convert(self, unit: BitRateUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: BitRateUnits = BitRateUnits.BitPerSecond) -> BitRateDto:
        """
        Get a new instance of BitRate DTO representing the current unit.

        :param hold_in_unit: The specific BitRate unit to store the BitRate value in the DTO representation.
        :type hold_in_unit: BitRateUnits
        :return: A new instance of BitRateDto.
        :rtype: BitRateDto
        """
        return BitRateDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: BitRateUnits = BitRateUnits.BitPerSecond):
        """
        Get a BitRate DTO JSON object representing the current unit.

        :param hold_in_unit: The specific BitRate unit to store the BitRate value in the DTO representation.
        :type hold_in_unit: BitRateUnits
        :return: JSON object represents BitRate DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "BitPerSecond"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(bit_rate_dto: BitRateDto):
        """
        Obtain a new instance of BitRate from a DTO unit object.

        :param bit_rate_dto: The BitRate DTO representation.
        :type bit_rate_dto: BitRateDto
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(bit_rate_dto.value, bit_rate_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of BitRate from a DTO unit json representation.

        :param data: The BitRate DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "BitPerSecond"}
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate.from_dto(BitRateDto.from_json(data))

    def __convert_from_base(self, from_unit: BitRateUnits) -> float:
        value = self._value
        
        if from_unit == BitRateUnits.BitPerSecond:
            return (value)
        
        if from_unit == BitRateUnits.BytePerSecond:
            return (value / 8)
        
        if from_unit == BitRateUnits.KilobitPerSecond:
            return ((value) / 1000.0)
        
        if from_unit == BitRateUnits.MegabitPerSecond:
            return ((value) / 1000000.0)
        
        if from_unit == BitRateUnits.GigabitPerSecond:
            return ((value) / 1000000000.0)
        
        if from_unit == BitRateUnits.TerabitPerSecond:
            return ((value) / 1000000000000.0)
        
        if from_unit == BitRateUnits.PetabitPerSecond:
            return ((value) / 1000000000000000.0)
        
        if from_unit == BitRateUnits.ExabitPerSecond:
            return ((value) / 1e+18)
        
        if from_unit == BitRateUnits.KibibitPerSecond:
            return ((value) / 1024)
        
        if from_unit == BitRateUnits.MebibitPerSecond:
            return ((value) / 1048576)
        
        if from_unit == BitRateUnits.GibibitPerSecond:
            return ((value) / 1073741824)
        
        if from_unit == BitRateUnits.TebibitPerSecond:
            return ((value) / 1099511627776)
        
        if from_unit == BitRateUnits.PebibitPerSecond:
            return ((value) / 1125899906842624)
        
        if from_unit == BitRateUnits.ExbibitPerSecond:
            return ((value) / 1152921504606846976)
        
        if from_unit == BitRateUnits.KilobytePerSecond:
            return ((value / 8) / 1000.0)
        
        if from_unit == BitRateUnits.MegabytePerSecond:
            return ((value / 8) / 1000000.0)
        
        if from_unit == BitRateUnits.GigabytePerSecond:
            return ((value / 8) / 1000000000.0)
        
        if from_unit == BitRateUnits.TerabytePerSecond:
            return ((value / 8) / 1000000000000.0)
        
        if from_unit == BitRateUnits.PetabytePerSecond:
            return ((value / 8) / 1000000000000000.0)
        
        if from_unit == BitRateUnits.ExabytePerSecond:
            return ((value / 8) / 1e+18)
        
        if from_unit == BitRateUnits.KibibytePerSecond:
            return ((value / 8) / 1024)
        
        if from_unit == BitRateUnits.MebibytePerSecond:
            return ((value / 8) / 1048576)
        
        if from_unit == BitRateUnits.GibibytePerSecond:
            return ((value / 8) / 1073741824)
        
        if from_unit == BitRateUnits.TebibytePerSecond:
            return ((value / 8) / 1099511627776)
        
        if from_unit == BitRateUnits.PebibytePerSecond:
            return ((value / 8) / 1125899906842624)
        
        if from_unit == BitRateUnits.ExbibytePerSecond:
            return ((value / 8) / 1152921504606846976)
        
        return None


    def __convert_to_base(self, value: float, to_unit: BitRateUnits) -> float:
        
        if to_unit == BitRateUnits.BitPerSecond:
            return (value)
        
        if to_unit == BitRateUnits.BytePerSecond:
            return (value * 8)
        
        if to_unit == BitRateUnits.KilobitPerSecond:
            return ((value) * 1000.0)
        
        if to_unit == BitRateUnits.MegabitPerSecond:
            return ((value) * 1000000.0)
        
        if to_unit == BitRateUnits.GigabitPerSecond:
            return ((value) * 1000000000.0)
        
        if to_unit == BitRateUnits.TerabitPerSecond:
            return ((value) * 1000000000000.0)
        
        if to_unit == BitRateUnits.PetabitPerSecond:
            return ((value) * 1000000000000000.0)
        
        if to_unit == BitRateUnits.ExabitPerSecond:
            return ((value) * 1e+18)
        
        if to_unit == BitRateUnits.KibibitPerSecond:
            return ((value) * 1024)
        
        if to_unit == BitRateUnits.MebibitPerSecond:
            return ((value) * 1048576)
        
        if to_unit == BitRateUnits.GibibitPerSecond:
            return ((value) * 1073741824)
        
        if to_unit == BitRateUnits.TebibitPerSecond:
            return ((value) * 1099511627776)
        
        if to_unit == BitRateUnits.PebibitPerSecond:
            return ((value) * 1125899906842624)
        
        if to_unit == BitRateUnits.ExbibitPerSecond:
            return ((value) * 1152921504606846976)
        
        if to_unit == BitRateUnits.KilobytePerSecond:
            return ((value * 8) * 1000.0)
        
        if to_unit == BitRateUnits.MegabytePerSecond:
            return ((value * 8) * 1000000.0)
        
        if to_unit == BitRateUnits.GigabytePerSecond:
            return ((value * 8) * 1000000000.0)
        
        if to_unit == BitRateUnits.TerabytePerSecond:
            return ((value * 8) * 1000000000000.0)
        
        if to_unit == BitRateUnits.PetabytePerSecond:
            return ((value * 8) * 1000000000000000.0)
        
        if to_unit == BitRateUnits.ExabytePerSecond:
            return ((value * 8) * 1e+18)
        
        if to_unit == BitRateUnits.KibibytePerSecond:
            return ((value * 8) * 1024)
        
        if to_unit == BitRateUnits.MebibytePerSecond:
            return ((value * 8) * 1048576)
        
        if to_unit == BitRateUnits.GibibytePerSecond:
            return ((value * 8) * 1073741824)
        
        if to_unit == BitRateUnits.TebibytePerSecond:
            return ((value * 8) * 1099511627776)
        
        if to_unit == BitRateUnits.PebibytePerSecond:
            return ((value * 8) * 1125899906842624)
        
        if to_unit == BitRateUnits.ExbibytePerSecond:
            return ((value * 8) * 1152921504606846976)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_bits_per_second(bits_per_second: float):
        """
        Create a new instance of BitRate from a value in bits_per_second.

        

        :param meters: The BitRate value in bits_per_second.
        :type bits_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(bits_per_second, BitRateUnits.BitPerSecond)

    
    @staticmethod
    def from_bytes_per_second(bytes_per_second: float):
        """
        Create a new instance of BitRate from a value in bytes_per_second.

        

        :param meters: The BitRate value in bytes_per_second.
        :type bytes_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(bytes_per_second, BitRateUnits.BytePerSecond)

    
    @staticmethod
    def from_kilobits_per_second(kilobits_per_second: float):
        """
        Create a new instance of BitRate from a value in kilobits_per_second.

        

        :param meters: The BitRate value in kilobits_per_second.
        :type kilobits_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(kilobits_per_second, BitRateUnits.KilobitPerSecond)

    
    @staticmethod
    def from_megabits_per_second(megabits_per_second: float):
        """
        Create a new instance of BitRate from a value in megabits_per_second.

        

        :param meters: The BitRate value in megabits_per_second.
        :type megabits_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(megabits_per_second, BitRateUnits.MegabitPerSecond)

    
    @staticmethod
    def from_gigabits_per_second(gigabits_per_second: float):
        """
        Create a new instance of BitRate from a value in gigabits_per_second.

        

        :param meters: The BitRate value in gigabits_per_second.
        :type gigabits_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(gigabits_per_second, BitRateUnits.GigabitPerSecond)

    
    @staticmethod
    def from_terabits_per_second(terabits_per_second: float):
        """
        Create a new instance of BitRate from a value in terabits_per_second.

        

        :param meters: The BitRate value in terabits_per_second.
        :type terabits_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(terabits_per_second, BitRateUnits.TerabitPerSecond)

    
    @staticmethod
    def from_petabits_per_second(petabits_per_second: float):
        """
        Create a new instance of BitRate from a value in petabits_per_second.

        

        :param meters: The BitRate value in petabits_per_second.
        :type petabits_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(petabits_per_second, BitRateUnits.PetabitPerSecond)

    
    @staticmethod
    def from_exabits_per_second(exabits_per_second: float):
        """
        Create a new instance of BitRate from a value in exabits_per_second.

        

        :param meters: The BitRate value in exabits_per_second.
        :type exabits_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(exabits_per_second, BitRateUnits.ExabitPerSecond)

    
    @staticmethod
    def from_kibibits_per_second(kibibits_per_second: float):
        """
        Create a new instance of BitRate from a value in kibibits_per_second.

        

        :param meters: The BitRate value in kibibits_per_second.
        :type kibibits_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(kibibits_per_second, BitRateUnits.KibibitPerSecond)

    
    @staticmethod
    def from_mebibits_per_second(mebibits_per_second: float):
        """
        Create a new instance of BitRate from a value in mebibits_per_second.

        

        :param meters: The BitRate value in mebibits_per_second.
        :type mebibits_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(mebibits_per_second, BitRateUnits.MebibitPerSecond)

    
    @staticmethod
    def from_gibibits_per_second(gibibits_per_second: float):
        """
        Create a new instance of BitRate from a value in gibibits_per_second.

        

        :param meters: The BitRate value in gibibits_per_second.
        :type gibibits_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(gibibits_per_second, BitRateUnits.GibibitPerSecond)

    
    @staticmethod
    def from_tebibits_per_second(tebibits_per_second: float):
        """
        Create a new instance of BitRate from a value in tebibits_per_second.

        

        :param meters: The BitRate value in tebibits_per_second.
        :type tebibits_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(tebibits_per_second, BitRateUnits.TebibitPerSecond)

    
    @staticmethod
    def from_pebibits_per_second(pebibits_per_second: float):
        """
        Create a new instance of BitRate from a value in pebibits_per_second.

        

        :param meters: The BitRate value in pebibits_per_second.
        :type pebibits_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(pebibits_per_second, BitRateUnits.PebibitPerSecond)

    
    @staticmethod
    def from_exbibits_per_second(exbibits_per_second: float):
        """
        Create a new instance of BitRate from a value in exbibits_per_second.

        

        :param meters: The BitRate value in exbibits_per_second.
        :type exbibits_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(exbibits_per_second, BitRateUnits.ExbibitPerSecond)

    
    @staticmethod
    def from_kilobytes_per_second(kilobytes_per_second: float):
        """
        Create a new instance of BitRate from a value in kilobytes_per_second.

        

        :param meters: The BitRate value in kilobytes_per_second.
        :type kilobytes_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(kilobytes_per_second, BitRateUnits.KilobytePerSecond)

    
    @staticmethod
    def from_megabytes_per_second(megabytes_per_second: float):
        """
        Create a new instance of BitRate from a value in megabytes_per_second.

        

        :param meters: The BitRate value in megabytes_per_second.
        :type megabytes_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(megabytes_per_second, BitRateUnits.MegabytePerSecond)

    
    @staticmethod
    def from_gigabytes_per_second(gigabytes_per_second: float):
        """
        Create a new instance of BitRate from a value in gigabytes_per_second.

        

        :param meters: The BitRate value in gigabytes_per_second.
        :type gigabytes_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(gigabytes_per_second, BitRateUnits.GigabytePerSecond)

    
    @staticmethod
    def from_terabytes_per_second(terabytes_per_second: float):
        """
        Create a new instance of BitRate from a value in terabytes_per_second.

        

        :param meters: The BitRate value in terabytes_per_second.
        :type terabytes_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(terabytes_per_second, BitRateUnits.TerabytePerSecond)

    
    @staticmethod
    def from_petabytes_per_second(petabytes_per_second: float):
        """
        Create a new instance of BitRate from a value in petabytes_per_second.

        

        :param meters: The BitRate value in petabytes_per_second.
        :type petabytes_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(petabytes_per_second, BitRateUnits.PetabytePerSecond)

    
    @staticmethod
    def from_exabytes_per_second(exabytes_per_second: float):
        """
        Create a new instance of BitRate from a value in exabytes_per_second.

        

        :param meters: The BitRate value in exabytes_per_second.
        :type exabytes_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(exabytes_per_second, BitRateUnits.ExabytePerSecond)

    
    @staticmethod
    def from_kibibytes_per_second(kibibytes_per_second: float):
        """
        Create a new instance of BitRate from a value in kibibytes_per_second.

        

        :param meters: The BitRate value in kibibytes_per_second.
        :type kibibytes_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(kibibytes_per_second, BitRateUnits.KibibytePerSecond)

    
    @staticmethod
    def from_mebibytes_per_second(mebibytes_per_second: float):
        """
        Create a new instance of BitRate from a value in mebibytes_per_second.

        

        :param meters: The BitRate value in mebibytes_per_second.
        :type mebibytes_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(mebibytes_per_second, BitRateUnits.MebibytePerSecond)

    
    @staticmethod
    def from_gibibytes_per_second(gibibytes_per_second: float):
        """
        Create a new instance of BitRate from a value in gibibytes_per_second.

        

        :param meters: The BitRate value in gibibytes_per_second.
        :type gibibytes_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(gibibytes_per_second, BitRateUnits.GibibytePerSecond)

    
    @staticmethod
    def from_tebibytes_per_second(tebibytes_per_second: float):
        """
        Create a new instance of BitRate from a value in tebibytes_per_second.

        

        :param meters: The BitRate value in tebibytes_per_second.
        :type tebibytes_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(tebibytes_per_second, BitRateUnits.TebibytePerSecond)

    
    @staticmethod
    def from_pebibytes_per_second(pebibytes_per_second: float):
        """
        Create a new instance of BitRate from a value in pebibytes_per_second.

        

        :param meters: The BitRate value in pebibytes_per_second.
        :type pebibytes_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(pebibytes_per_second, BitRateUnits.PebibytePerSecond)

    
    @staticmethod
    def from_exbibytes_per_second(exbibytes_per_second: float):
        """
        Create a new instance of BitRate from a value in exbibytes_per_second.

        

        :param meters: The BitRate value in exbibytes_per_second.
        :type exbibytes_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(exbibytes_per_second, BitRateUnits.ExbibytePerSecond)

    
    @property
    def bits_per_second(self) -> float:
        """
        
        """
        if self.__bits_per_second != None:
            return self.__bits_per_second
        self.__bits_per_second = self.__convert_from_base(BitRateUnits.BitPerSecond)
        return self.__bits_per_second

    
    @property
    def bytes_per_second(self) -> float:
        """
        
        """
        if self.__bytes_per_second != None:
            return self.__bytes_per_second
        self.__bytes_per_second = self.__convert_from_base(BitRateUnits.BytePerSecond)
        return self.__bytes_per_second

    
    @property
    def kilobits_per_second(self) -> float:
        """
        
        """
        if self.__kilobits_per_second != None:
            return self.__kilobits_per_second
        self.__kilobits_per_second = self.__convert_from_base(BitRateUnits.KilobitPerSecond)
        return self.__kilobits_per_second

    
    @property
    def megabits_per_second(self) -> float:
        """
        
        """
        if self.__megabits_per_second != None:
            return self.__megabits_per_second
        self.__megabits_per_second = self.__convert_from_base(BitRateUnits.MegabitPerSecond)
        return self.__megabits_per_second

    
    @property
    def gigabits_per_second(self) -> float:
        """
        
        """
        if self.__gigabits_per_second != None:
            return self.__gigabits_per_second
        self.__gigabits_per_second = self.__convert_from_base(BitRateUnits.GigabitPerSecond)
        return self.__gigabits_per_second

    
    @property
    def terabits_per_second(self) -> float:
        """
        
        """
        if self.__terabits_per_second != None:
            return self.__terabits_per_second
        self.__terabits_per_second = self.__convert_from_base(BitRateUnits.TerabitPerSecond)
        return self.__terabits_per_second

    
    @property
    def petabits_per_second(self) -> float:
        """
        
        """
        if self.__petabits_per_second != None:
            return self.__petabits_per_second
        self.__petabits_per_second = self.__convert_from_base(BitRateUnits.PetabitPerSecond)
        return self.__petabits_per_second

    
    @property
    def exabits_per_second(self) -> float:
        """
        
        """
        if self.__exabits_per_second != None:
            return self.__exabits_per_second
        self.__exabits_per_second = self.__convert_from_base(BitRateUnits.ExabitPerSecond)
        return self.__exabits_per_second

    
    @property
    def kibibits_per_second(self) -> float:
        """
        
        """
        if self.__kibibits_per_second != None:
            return self.__kibibits_per_second
        self.__kibibits_per_second = self.__convert_from_base(BitRateUnits.KibibitPerSecond)
        return self.__kibibits_per_second

    
    @property
    def mebibits_per_second(self) -> float:
        """
        
        """
        if self.__mebibits_per_second != None:
            return self.__mebibits_per_second
        self.__mebibits_per_second = self.__convert_from_base(BitRateUnits.MebibitPerSecond)
        return self.__mebibits_per_second

    
    @property
    def gibibits_per_second(self) -> float:
        """
        
        """
        if self.__gibibits_per_second != None:
            return self.__gibibits_per_second
        self.__gibibits_per_second = self.__convert_from_base(BitRateUnits.GibibitPerSecond)
        return self.__gibibits_per_second

    
    @property
    def tebibits_per_second(self) -> float:
        """
        
        """
        if self.__tebibits_per_second != None:
            return self.__tebibits_per_second
        self.__tebibits_per_second = self.__convert_from_base(BitRateUnits.TebibitPerSecond)
        return self.__tebibits_per_second

    
    @property
    def pebibits_per_second(self) -> float:
        """
        
        """
        if self.__pebibits_per_second != None:
            return self.__pebibits_per_second
        self.__pebibits_per_second = self.__convert_from_base(BitRateUnits.PebibitPerSecond)
        return self.__pebibits_per_second

    
    @property
    def exbibits_per_second(self) -> float:
        """
        
        """
        if self.__exbibits_per_second != None:
            return self.__exbibits_per_second
        self.__exbibits_per_second = self.__convert_from_base(BitRateUnits.ExbibitPerSecond)
        return self.__exbibits_per_second

    
    @property
    def kilobytes_per_second(self) -> float:
        """
        
        """
        if self.__kilobytes_per_second != None:
            return self.__kilobytes_per_second
        self.__kilobytes_per_second = self.__convert_from_base(BitRateUnits.KilobytePerSecond)
        return self.__kilobytes_per_second

    
    @property
    def megabytes_per_second(self) -> float:
        """
        
        """
        if self.__megabytes_per_second != None:
            return self.__megabytes_per_second
        self.__megabytes_per_second = self.__convert_from_base(BitRateUnits.MegabytePerSecond)
        return self.__megabytes_per_second

    
    @property
    def gigabytes_per_second(self) -> float:
        """
        
        """
        if self.__gigabytes_per_second != None:
            return self.__gigabytes_per_second
        self.__gigabytes_per_second = self.__convert_from_base(BitRateUnits.GigabytePerSecond)
        return self.__gigabytes_per_second

    
    @property
    def terabytes_per_second(self) -> float:
        """
        
        """
        if self.__terabytes_per_second != None:
            return self.__terabytes_per_second
        self.__terabytes_per_second = self.__convert_from_base(BitRateUnits.TerabytePerSecond)
        return self.__terabytes_per_second

    
    @property
    def petabytes_per_second(self) -> float:
        """
        
        """
        if self.__petabytes_per_second != None:
            return self.__petabytes_per_second
        self.__petabytes_per_second = self.__convert_from_base(BitRateUnits.PetabytePerSecond)
        return self.__petabytes_per_second

    
    @property
    def exabytes_per_second(self) -> float:
        """
        
        """
        if self.__exabytes_per_second != None:
            return self.__exabytes_per_second
        self.__exabytes_per_second = self.__convert_from_base(BitRateUnits.ExabytePerSecond)
        return self.__exabytes_per_second

    
    @property
    def kibibytes_per_second(self) -> float:
        """
        
        """
        if self.__kibibytes_per_second != None:
            return self.__kibibytes_per_second
        self.__kibibytes_per_second = self.__convert_from_base(BitRateUnits.KibibytePerSecond)
        return self.__kibibytes_per_second

    
    @property
    def mebibytes_per_second(self) -> float:
        """
        
        """
        if self.__mebibytes_per_second != None:
            return self.__mebibytes_per_second
        self.__mebibytes_per_second = self.__convert_from_base(BitRateUnits.MebibytePerSecond)
        return self.__mebibytes_per_second

    
    @property
    def gibibytes_per_second(self) -> float:
        """
        
        """
        if self.__gibibytes_per_second != None:
            return self.__gibibytes_per_second
        self.__gibibytes_per_second = self.__convert_from_base(BitRateUnits.GibibytePerSecond)
        return self.__gibibytes_per_second

    
    @property
    def tebibytes_per_second(self) -> float:
        """
        
        """
        if self.__tebibytes_per_second != None:
            return self.__tebibytes_per_second
        self.__tebibytes_per_second = self.__convert_from_base(BitRateUnits.TebibytePerSecond)
        return self.__tebibytes_per_second

    
    @property
    def pebibytes_per_second(self) -> float:
        """
        
        """
        if self.__pebibytes_per_second != None:
            return self.__pebibytes_per_second
        self.__pebibytes_per_second = self.__convert_from_base(BitRateUnits.PebibytePerSecond)
        return self.__pebibytes_per_second

    
    @property
    def exbibytes_per_second(self) -> float:
        """
        
        """
        if self.__exbibytes_per_second != None:
            return self.__exbibytes_per_second
        self.__exbibytes_per_second = self.__convert_from_base(BitRateUnits.ExbibytePerSecond)
        return self.__exbibytes_per_second

    
    def to_string(self, unit: BitRateUnits = BitRateUnits.BitPerSecond, fractional_digits: int = None) -> str:
        """
        Format the BitRate to a string.
        
        Note: the default format for BitRate is BitPerSecond.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the BitRate. Default is 'BitPerSecond'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == BitRateUnits.BitPerSecond:
            return f"""{super()._truncate_fraction_digits(self.bits_per_second, fractional_digits)} bit/s"""
        
        if unit == BitRateUnits.BytePerSecond:
            return f"""{super()._truncate_fraction_digits(self.bytes_per_second, fractional_digits)} B/s"""
        
        if unit == BitRateUnits.KilobitPerSecond:
            return f"""{super()._truncate_fraction_digits(self.kilobits_per_second, fractional_digits)} kbit/s"""
        
        if unit == BitRateUnits.MegabitPerSecond:
            return f"""{super()._truncate_fraction_digits(self.megabits_per_second, fractional_digits)} Mbit/s"""
        
        if unit == BitRateUnits.GigabitPerSecond:
            return f"""{super()._truncate_fraction_digits(self.gigabits_per_second, fractional_digits)} Gbit/s"""
        
        if unit == BitRateUnits.TerabitPerSecond:
            return f"""{super()._truncate_fraction_digits(self.terabits_per_second, fractional_digits)} Tbit/s"""
        
        if unit == BitRateUnits.PetabitPerSecond:
            return f"""{super()._truncate_fraction_digits(self.petabits_per_second, fractional_digits)} Pbit/s"""
        
        if unit == BitRateUnits.ExabitPerSecond:
            return f"""{super()._truncate_fraction_digits(self.exabits_per_second, fractional_digits)} Ebit/s"""
        
        if unit == BitRateUnits.KibibitPerSecond:
            return f"""{super()._truncate_fraction_digits(self.kibibits_per_second, fractional_digits)} KiBbit/s"""
        
        if unit == BitRateUnits.MebibitPerSecond:
            return f"""{super()._truncate_fraction_digits(self.mebibits_per_second, fractional_digits)} MiBbit/s"""
        
        if unit == BitRateUnits.GibibitPerSecond:
            return f"""{super()._truncate_fraction_digits(self.gibibits_per_second, fractional_digits)} GiBbit/s"""
        
        if unit == BitRateUnits.TebibitPerSecond:
            return f"""{super()._truncate_fraction_digits(self.tebibits_per_second, fractional_digits)} TiBbit/s"""
        
        if unit == BitRateUnits.PebibitPerSecond:
            return f"""{super()._truncate_fraction_digits(self.pebibits_per_second, fractional_digits)} PiBbit/s"""
        
        if unit == BitRateUnits.ExbibitPerSecond:
            return f"""{super()._truncate_fraction_digits(self.exbibits_per_second, fractional_digits)} EiBbit/s"""
        
        if unit == BitRateUnits.KilobytePerSecond:
            return f"""{super()._truncate_fraction_digits(self.kilobytes_per_second, fractional_digits)} kB/s"""
        
        if unit == BitRateUnits.MegabytePerSecond:
            return f"""{super()._truncate_fraction_digits(self.megabytes_per_second, fractional_digits)} MB/s"""
        
        if unit == BitRateUnits.GigabytePerSecond:
            return f"""{super()._truncate_fraction_digits(self.gigabytes_per_second, fractional_digits)} GB/s"""
        
        if unit == BitRateUnits.TerabytePerSecond:
            return f"""{super()._truncate_fraction_digits(self.terabytes_per_second, fractional_digits)} TB/s"""
        
        if unit == BitRateUnits.PetabytePerSecond:
            return f"""{super()._truncate_fraction_digits(self.petabytes_per_second, fractional_digits)} PB/s"""
        
        if unit == BitRateUnits.ExabytePerSecond:
            return f"""{super()._truncate_fraction_digits(self.exabytes_per_second, fractional_digits)} EB/s"""
        
        if unit == BitRateUnits.KibibytePerSecond:
            return f"""{super()._truncate_fraction_digits(self.kibibytes_per_second, fractional_digits)} KiBB/s"""
        
        if unit == BitRateUnits.MebibytePerSecond:
            return f"""{super()._truncate_fraction_digits(self.mebibytes_per_second, fractional_digits)} MiBB/s"""
        
        if unit == BitRateUnits.GibibytePerSecond:
            return f"""{super()._truncate_fraction_digits(self.gibibytes_per_second, fractional_digits)} GiBB/s"""
        
        if unit == BitRateUnits.TebibytePerSecond:
            return f"""{super()._truncate_fraction_digits(self.tebibytes_per_second, fractional_digits)} TiBB/s"""
        
        if unit == BitRateUnits.PebibytePerSecond:
            return f"""{super()._truncate_fraction_digits(self.pebibytes_per_second, fractional_digits)} PiBB/s"""
        
        if unit == BitRateUnits.ExbibytePerSecond:
            return f"""{super()._truncate_fraction_digits(self.exbibytes_per_second, fractional_digits)} EiBB/s"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: BitRateUnits = BitRateUnits.BitPerSecond) -> str:
        """
        Get BitRate unit abbreviation.
        Note! the default abbreviation for BitRate is BitPerSecond.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == BitRateUnits.BitPerSecond:
            return """bit/s"""
        
        if unit_abbreviation == BitRateUnits.BytePerSecond:
            return """B/s"""
        
        if unit_abbreviation == BitRateUnits.KilobitPerSecond:
            return """kbit/s"""
        
        if unit_abbreviation == BitRateUnits.MegabitPerSecond:
            return """Mbit/s"""
        
        if unit_abbreviation == BitRateUnits.GigabitPerSecond:
            return """Gbit/s"""
        
        if unit_abbreviation == BitRateUnits.TerabitPerSecond:
            return """Tbit/s"""
        
        if unit_abbreviation == BitRateUnits.PetabitPerSecond:
            return """Pbit/s"""
        
        if unit_abbreviation == BitRateUnits.ExabitPerSecond:
            return """Ebit/s"""
        
        if unit_abbreviation == BitRateUnits.KibibitPerSecond:
            return """KiBbit/s"""
        
        if unit_abbreviation == BitRateUnits.MebibitPerSecond:
            return """MiBbit/s"""
        
        if unit_abbreviation == BitRateUnits.GibibitPerSecond:
            return """GiBbit/s"""
        
        if unit_abbreviation == BitRateUnits.TebibitPerSecond:
            return """TiBbit/s"""
        
        if unit_abbreviation == BitRateUnits.PebibitPerSecond:
            return """PiBbit/s"""
        
        if unit_abbreviation == BitRateUnits.ExbibitPerSecond:
            return """EiBbit/s"""
        
        if unit_abbreviation == BitRateUnits.KilobytePerSecond:
            return """kB/s"""
        
        if unit_abbreviation == BitRateUnits.MegabytePerSecond:
            return """MB/s"""
        
        if unit_abbreviation == BitRateUnits.GigabytePerSecond:
            return """GB/s"""
        
        if unit_abbreviation == BitRateUnits.TerabytePerSecond:
            return """TB/s"""
        
        if unit_abbreviation == BitRateUnits.PetabytePerSecond:
            return """PB/s"""
        
        if unit_abbreviation == BitRateUnits.ExabytePerSecond:
            return """EB/s"""
        
        if unit_abbreviation == BitRateUnits.KibibytePerSecond:
            return """KiBB/s"""
        
        if unit_abbreviation == BitRateUnits.MebibytePerSecond:
            return """MiBB/s"""
        
        if unit_abbreviation == BitRateUnits.GibibytePerSecond:
            return """GiBB/s"""
        
        if unit_abbreviation == BitRateUnits.TebibytePerSecond:
            return """TiBB/s"""
        
        if unit_abbreviation == BitRateUnits.PebibytePerSecond:
            return """PiBB/s"""
        
        if unit_abbreviation == BitRateUnits.ExbibytePerSecond:
            return """EiBB/s"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class MolalityUnits(Enum):
        """
            MolalityUnits enumeration
        """
        
        MolePerKilogram = 'MolePerKilogram'
        """
            
        """
        
        MolePerGram = 'MolePerGram'
        """
            
        """
        
        MillimolePerKilogram = 'MillimolePerKilogram'
        """
            
        """
        

class MolalityDto:
    """
    A DTO representation of a Molality

    Attributes:
        value (float): The value of the Molality.
        unit (MolalityUnits): The specific unit that the Molality value is representing.
    """

    def __init__(self, value: float, unit: MolalityUnits):
        """
        Create a new DTO representation of a Molality

        Parameters:
            value (float): The value of the Molality.
            unit (MolalityUnits): The specific unit that the Molality value is representing.
        """
        self.value: float = value
        """
        The value of the Molality
        """
        self.unit: MolalityUnits = unit
        """
        The specific unit that the Molality value is representing
        """

    def to_json(self):
        """
        Get a Molality DTO JSON object representing the current unit.

        :return: JSON object represents Molality DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "MolePerKilogram"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Molality DTO from a json representation.

        :param data: The Molality DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "MolePerKilogram"}
        :return: A new instance of MolalityDto.
        :rtype: MolalityDto
        """
        return MolalityDto(value=data["value"], unit=MolalityUnits(data["unit"]))


class Molality(AbstractMeasure):
    """
    Molality is a measure of the amount of solute in a solution relative to a given mass of solvent.

    Args:
        value (float): The value.
        from_unit (MolalityUnits): The Molality unit to create from, The default unit is MolePerKilogram
    """
    def __init__(self, value: float, from_unit: MolalityUnits = MolalityUnits.MolePerKilogram):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__moles_per_kilogram = None
        
        self.__moles_per_gram = None
        
        self.__millimoles_per_kilogram = None
        

    def convert(self, unit: MolalityUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: MolalityUnits = MolalityUnits.MolePerKilogram) -> MolalityDto:
        """
        Get a new instance of Molality DTO representing the current unit.

        :param hold_in_unit: The specific Molality unit to store the Molality value in the DTO representation.
        :type hold_in_unit: MolalityUnits
        :return: A new instance of MolalityDto.
        :rtype: MolalityDto
        """
        return MolalityDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: MolalityUnits = MolalityUnits.MolePerKilogram):
        """
        Get a Molality DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Molality unit to store the Molality value in the DTO representation.
        :type hold_in_unit: MolalityUnits
        :return: JSON object represents Molality DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "MolePerKilogram"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(molality_dto: MolalityDto):
        """
        Obtain a new instance of Molality from a DTO unit object.

        :param molality_dto: The Molality DTO representation.
        :type molality_dto: MolalityDto
        :return: A new instance of Molality.
        :rtype: Molality
        """
        return Molality(molality_dto.value, molality_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Molality from a DTO unit json representation.

        :param data: The Molality DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "MolePerKilogram"}
        :return: A new instance of Molality.
        :rtype: Molality
        """
        return Molality.from_dto(MolalityDto.from_json(data))

    def __convert_from_base(self, from_unit: MolalityUnits) -> float:
        value = self._value
        
        if from_unit == MolalityUnits.MolePerKilogram:
            return (value)
        
        if from_unit == MolalityUnits.MolePerGram:
            return (value * 1e-3)
        
        if from_unit == MolalityUnits.MillimolePerKilogram:
            return ((value) / 0.001)
        
        return None


    def __convert_to_base(self, value: float, to_unit: MolalityUnits) -> float:
        
        if to_unit == MolalityUnits.MolePerKilogram:
            return (value)
        
        if to_unit == MolalityUnits.MolePerGram:
            return (value / 1e-3)
        
        if to_unit == MolalityUnits.MillimolePerKilogram:
            return ((value) * 0.001)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_moles_per_kilogram(moles_per_kilogram: float):
        """
        Create a new instance of Molality from a value in moles_per_kilogram.

        

        :param meters: The Molality value in moles_per_kilogram.
        :type moles_per_kilogram: float
        :return: A new instance of Molality.
        :rtype: Molality
        """
        return Molality(moles_per_kilogram, MolalityUnits.MolePerKilogram)

    
    @staticmethod
    def from_moles_per_gram(moles_per_gram: float):
        """
        Create a new instance of Molality from a value in moles_per_gram.

        

        :param meters: The Molality value in moles_per_gram.
        :type moles_per_gram: float
        :return: A new instance of Molality.
        :rtype: Molality
        """
        return Molality(moles_per_gram, MolalityUnits.MolePerGram)

    
    @staticmethod
    def from_millimoles_per_kilogram(millimoles_per_kilogram: float):
        """
        Create a new instance of Molality from a value in millimoles_per_kilogram.

        

        :param meters: The Molality value in millimoles_per_kilogram.
        :type millimoles_per_kilogram: float
        :return: A new instance of Molality.
        :rtype: Molality
        """
        return Molality(millimoles_per_kilogram, MolalityUnits.MillimolePerKilogram)

    
    @property
    def moles_per_kilogram(self) -> float:
        """
        
        """
        if self.__moles_per_kilogram != None:
            return self.__moles_per_kilogram
        self.__moles_per_kilogram = self.__convert_from_base(MolalityUnits.MolePerKilogram)
        return self.__moles_per_kilogram

    
    @property
    def moles_per_gram(self) -> float:
        """
        
        """
        if self.__moles_per_gram != None:
            return self.__moles_per_gram
        self.__moles_per_gram = self.__convert_from_base(MolalityUnits.MolePerGram)
        return self.__moles_per_gram

    
    @property
    def millimoles_per_kilogram(self) -> float:
        """
        
        """
        if self.__millimoles_per_kilogram != None:
            return self.__millimoles_per_kilogram
        self.__millimoles_per_kilogram = self.__convert_from_base(MolalityUnits.MillimolePerKilogram)
        return self.__millimoles_per_kilogram

    
    def to_string(self, unit: MolalityUnits = MolalityUnits.MolePerKilogram, fractional_digits: int = None) -> str:
        """
        Format the Molality to a string.
        
        Note: the default format for Molality is MolePerKilogram.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Molality. Default is 'MolePerKilogram'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == MolalityUnits.MolePerKilogram:
            return f"""{super()._truncate_fraction_digits(self.moles_per_kilogram, fractional_digits)} mol/kg"""
        
        if unit == MolalityUnits.MolePerGram:
            return f"""{super()._truncate_fraction_digits(self.moles_per_gram, fractional_digits)} mol/g"""
        
        if unit == MolalityUnits.MillimolePerKilogram:
            return f"""{super()._truncate_fraction_digits(self.millimoles_per_kilogram, fractional_digits)} mmol/kg"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: MolalityUnits = MolalityUnits.MolePerKilogram) -> str:
        """
        Get Molality unit abbreviation.
        Note! the default abbreviation for Molality is MolePerKilogram.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == MolalityUnits.MolePerKilogram:
            return """mol/kg"""
        
        if unit_abbreviation == MolalityUnits.MolePerGram:
            return """mol/g"""
        
        if unit_abbreviation == MolalityUnits.MillimolePerKilogram:
            return """mmol/kg"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ElectricPotentialAcUnits(Enum):
        """
            ElectricPotentialAcUnits enumeration
        """
        
        VoltAc = 'VoltAc'
        """
            
        """
        
        MicrovoltAc = 'MicrovoltAc'
        """
            
        """
        
        MillivoltAc = 'MillivoltAc'
        """
            
        """
        
        KilovoltAc = 'KilovoltAc'
        """
            
        """
        
        MegavoltAc = 'MegavoltAc'
        """
            
        """
        

class ElectricPotentialAcDto:
    """
    A DTO representation of a ElectricPotentialAc

    Attributes:
        value (float): The value of the ElectricPotentialAc.
        unit (ElectricPotentialAcUnits): The specific unit that the ElectricPotentialAc value is representing.
    """

    def __init__(self, value: float, unit: ElectricPotentialAcUnits):
        """
        Create a new DTO representation of a ElectricPotentialAc

        Parameters:
            value (float): The value of the ElectricPotentialAc.
            unit (ElectricPotentialAcUnits): The specific unit that the ElectricPotentialAc value is representing.
        """
        self.value: float = value
        """
        The value of the ElectricPotentialAc
        """
        self.unit: ElectricPotentialAcUnits = unit
        """
        The specific unit that the ElectricPotentialAc value is representing
        """

    def to_json(self):
        """
        Get a ElectricPotentialAc DTO JSON object representing the current unit.

        :return: JSON object represents ElectricPotentialAc DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "VoltAc"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ElectricPotentialAc DTO from a json representation.

        :param data: The ElectricPotentialAc DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "VoltAc"}
        :return: A new instance of ElectricPotentialAcDto.
        :rtype: ElectricPotentialAcDto
        """
        return ElectricPotentialAcDto(value=data["value"], unit=ElectricPotentialAcUnits(data["unit"]))


class ElectricPotentialAc(AbstractMeasure):
    """
    The Electric Potential of a system known to use Alternating Current.

    Args:
        value (float): The value.
        from_unit (ElectricPotentialAcUnits): The ElectricPotentialAc unit to create from, The default unit is VoltAc
    """
    def __init__(self, value: float, from_unit: ElectricPotentialAcUnits = ElectricPotentialAcUnits.VoltAc):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__volts_ac = None
        
        self.__microvolts_ac = None
        
        self.__millivolts_ac = None
        
        self.__kilovolts_ac = None
        
        self.__megavolts_ac = None
        

    def convert(self, unit: ElectricPotentialAcUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ElectricPotentialAcUnits = ElectricPotentialAcUnits.VoltAc) -> ElectricPotentialAcDto:
        """
        Get a new instance of ElectricPotentialAc DTO representing the current unit.

        :param hold_in_unit: The specific ElectricPotentialAc unit to store the ElectricPotentialAc value in the DTO representation.
        :type hold_in_unit: ElectricPotentialAcUnits
        :return: A new instance of ElectricPotentialAcDto.
        :rtype: ElectricPotentialAcDto
        """
        return ElectricPotentialAcDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ElectricPotentialAcUnits = ElectricPotentialAcUnits.VoltAc):
        """
        Get a ElectricPotentialAc DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ElectricPotentialAc unit to store the ElectricPotentialAc value in the DTO representation.
        :type hold_in_unit: ElectricPotentialAcUnits
        :return: JSON object represents ElectricPotentialAc DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "VoltAc"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(electric_potential_ac_dto: ElectricPotentialAcDto):
        """
        Obtain a new instance of ElectricPotentialAc from a DTO unit object.

        :param electric_potential_ac_dto: The ElectricPotentialAc DTO representation.
        :type electric_potential_ac_dto: ElectricPotentialAcDto
        :return: A new instance of ElectricPotentialAc.
        :rtype: ElectricPotentialAc
        """
        return ElectricPotentialAc(electric_potential_ac_dto.value, electric_potential_ac_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ElectricPotentialAc from a DTO unit json representation.

        :param data: The ElectricPotentialAc DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "VoltAc"}
        :return: A new instance of ElectricPotentialAc.
        :rtype: ElectricPotentialAc
        """
        return ElectricPotentialAc.from_dto(ElectricPotentialAcDto.from_json(data))

    def __convert_from_base(self, from_unit: ElectricPotentialAcUnits) -> float:
        value = self._value
        
        if from_unit == ElectricPotentialAcUnits.VoltAc:
            return (value)
        
        if from_unit == ElectricPotentialAcUnits.MicrovoltAc:
            return ((value) / 1e-06)
        
        if from_unit == ElectricPotentialAcUnits.MillivoltAc:
            return ((value) / 0.001)
        
        if from_unit == ElectricPotentialAcUnits.KilovoltAc:
            return ((value) / 1000.0)
        
        if from_unit == ElectricPotentialAcUnits.MegavoltAc:
            return ((value) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ElectricPotentialAcUnits) -> float:
        
        if to_unit == ElectricPotentialAcUnits.VoltAc:
            return (value)
        
        if to_unit == ElectricPotentialAcUnits.MicrovoltAc:
            return ((value) * 1e-06)
        
        if to_unit == ElectricPotentialAcUnits.MillivoltAc:
            return ((value) * 0.001)
        
        if to_unit == ElectricPotentialAcUnits.KilovoltAc:
            return ((value) * 1000.0)
        
        if to_unit == ElectricPotentialAcUnits.MegavoltAc:
            return ((value) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_volts_ac(volts_ac: float):
        """
        Create a new instance of ElectricPotentialAc from a value in volts_ac.

        

        :param meters: The ElectricPotentialAc value in volts_ac.
        :type volts_ac: float
        :return: A new instance of ElectricPotentialAc.
        :rtype: ElectricPotentialAc
        """
        return ElectricPotentialAc(volts_ac, ElectricPotentialAcUnits.VoltAc)

    
    @staticmethod
    def from_microvolts_ac(microvolts_ac: float):
        """
        Create a new instance of ElectricPotentialAc from a value in microvolts_ac.

        

        :param meters: The ElectricPotentialAc value in microvolts_ac.
        :type microvolts_ac: float
        :return: A new instance of ElectricPotentialAc.
        :rtype: ElectricPotentialAc
        """
        return ElectricPotentialAc(microvolts_ac, ElectricPotentialAcUnits.MicrovoltAc)

    
    @staticmethod
    def from_millivolts_ac(millivolts_ac: float):
        """
        Create a new instance of ElectricPotentialAc from a value in millivolts_ac.

        

        :param meters: The ElectricPotentialAc value in millivolts_ac.
        :type millivolts_ac: float
        :return: A new instance of ElectricPotentialAc.
        :rtype: ElectricPotentialAc
        """
        return ElectricPotentialAc(millivolts_ac, ElectricPotentialAcUnits.MillivoltAc)

    
    @staticmethod
    def from_kilovolts_ac(kilovolts_ac: float):
        """
        Create a new instance of ElectricPotentialAc from a value in kilovolts_ac.

        

        :param meters: The ElectricPotentialAc value in kilovolts_ac.
        :type kilovolts_ac: float
        :return: A new instance of ElectricPotentialAc.
        :rtype: ElectricPotentialAc
        """
        return ElectricPotentialAc(kilovolts_ac, ElectricPotentialAcUnits.KilovoltAc)

    
    @staticmethod
    def from_megavolts_ac(megavolts_ac: float):
        """
        Create a new instance of ElectricPotentialAc from a value in megavolts_ac.

        

        :param meters: The ElectricPotentialAc value in megavolts_ac.
        :type megavolts_ac: float
        :return: A new instance of ElectricPotentialAc.
        :rtype: ElectricPotentialAc
        """
        return ElectricPotentialAc(megavolts_ac, ElectricPotentialAcUnits.MegavoltAc)

    
    @property
    def volts_ac(self) -> float:
        """
        
        """
        if self.__volts_ac != None:
            return self.__volts_ac
        self.__volts_ac = self.__convert_from_base(ElectricPotentialAcUnits.VoltAc)
        return self.__volts_ac

    
    @property
    def microvolts_ac(self) -> float:
        """
        
        """
        if self.__microvolts_ac != None:
            return self.__microvolts_ac
        self.__microvolts_ac = self.__convert_from_base(ElectricPotentialAcUnits.MicrovoltAc)
        return self.__microvolts_ac

    
    @property
    def millivolts_ac(self) -> float:
        """
        
        """
        if self.__millivolts_ac != None:
            return self.__millivolts_ac
        self.__millivolts_ac = self.__convert_from_base(ElectricPotentialAcUnits.MillivoltAc)
        return self.__millivolts_ac

    
    @property
    def kilovolts_ac(self) -> float:
        """
        
        """
        if self.__kilovolts_ac != None:
            return self.__kilovolts_ac
        self.__kilovolts_ac = self.__convert_from_base(ElectricPotentialAcUnits.KilovoltAc)
        return self.__kilovolts_ac

    
    @property
    def megavolts_ac(self) -> float:
        """
        
        """
        if self.__megavolts_ac != None:
            return self.__megavolts_ac
        self.__megavolts_ac = self.__convert_from_base(ElectricPotentialAcUnits.MegavoltAc)
        return self.__megavolts_ac

    
    def to_string(self, unit: ElectricPotentialAcUnits = ElectricPotentialAcUnits.VoltAc, fractional_digits: int = None) -> str:
        """
        Format the ElectricPotentialAc to a string.
        
        Note: the default format for ElectricPotentialAc is VoltAc.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ElectricPotentialAc. Default is 'VoltAc'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ElectricPotentialAcUnits.VoltAc:
            return f"""{super()._truncate_fraction_digits(self.volts_ac, fractional_digits)} Vac"""
        
        if unit == ElectricPotentialAcUnits.MicrovoltAc:
            return f"""{super()._truncate_fraction_digits(self.microvolts_ac, fractional_digits)} μVac"""
        
        if unit == ElectricPotentialAcUnits.MillivoltAc:
            return f"""{super()._truncate_fraction_digits(self.millivolts_ac, fractional_digits)} mVac"""
        
        if unit == ElectricPotentialAcUnits.KilovoltAc:
            return f"""{super()._truncate_fraction_digits(self.kilovolts_ac, fractional_digits)} kVac"""
        
        if unit == ElectricPotentialAcUnits.MegavoltAc:
            return f"""{super()._truncate_fraction_digits(self.megavolts_ac, fractional_digits)} MVac"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ElectricPotentialAcUnits = ElectricPotentialAcUnits.VoltAc) -> str:
        """
        Get ElectricPotentialAc unit abbreviation.
        Note! the default abbreviation for ElectricPotentialAc is VoltAc.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ElectricPotentialAcUnits.VoltAc:
            return """Vac"""
        
        if unit_abbreviation == ElectricPotentialAcUnits.MicrovoltAc:
            return """μVac"""
        
        if unit_abbreviation == ElectricPotentialAcUnits.MillivoltAc:
            return """mVac"""
        
        if unit_abbreviation == ElectricPotentialAcUnits.KilovoltAc:
            return """kVac"""
        
        if unit_abbreviation == ElectricPotentialAcUnits.MegavoltAc:
            return """MVac"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class IrradiationUnits(Enum):
        """
            IrradiationUnits enumeration
        """
        
        JoulePerSquareMeter = 'JoulePerSquareMeter'
        """
            
        """
        
        JoulePerSquareCentimeter = 'JoulePerSquareCentimeter'
        """
            
        """
        
        JoulePerSquareMillimeter = 'JoulePerSquareMillimeter'
        """
            
        """
        
        WattHourPerSquareMeter = 'WattHourPerSquareMeter'
        """
            
        """
        
        BtuPerSquareFoot = 'BtuPerSquareFoot'
        """
            
        """
        
        KilojoulePerSquareMeter = 'KilojoulePerSquareMeter'
        """
            
        """
        
        MillijoulePerSquareCentimeter = 'MillijoulePerSquareCentimeter'
        """
            
        """
        
        KilowattHourPerSquareMeter = 'KilowattHourPerSquareMeter'
        """
            
        """
        
        KilobtuPerSquareFoot = 'KilobtuPerSquareFoot'
        """
            
        """
        

class IrradiationDto:
    """
    A DTO representation of a Irradiation

    Attributes:
        value (float): The value of the Irradiation.
        unit (IrradiationUnits): The specific unit that the Irradiation value is representing.
    """

    def __init__(self, value: float, unit: IrradiationUnits):
        """
        Create a new DTO representation of a Irradiation

        Parameters:
            value (float): The value of the Irradiation.
            unit (IrradiationUnits): The specific unit that the Irradiation value is representing.
        """
        self.value: float = value
        """
        The value of the Irradiation
        """
        self.unit: IrradiationUnits = unit
        """
        The specific unit that the Irradiation value is representing
        """

    def to_json(self):
        """
        Get a Irradiation DTO JSON object representing the current unit.

        :return: JSON object represents Irradiation DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "JoulePerSquareMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Irradiation DTO from a json representation.

        :param data: The Irradiation DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "JoulePerSquareMeter"}
        :return: A new instance of IrradiationDto.
        :rtype: IrradiationDto
        """
        return IrradiationDto(value=data["value"], unit=IrradiationUnits(data["unit"]))


class Irradiation(AbstractMeasure):
    """
    Irradiation is the process by which an object is exposed to radiation. The exposure can originate from various sources, including natural sources.

    Args:
        value (float): The value.
        from_unit (IrradiationUnits): The Irradiation unit to create from, The default unit is JoulePerSquareMeter
    """
    def __init__(self, value: float, from_unit: IrradiationUnits = IrradiationUnits.JoulePerSquareMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__joules_per_square_meter = None
        
        self.__joules_per_square_centimeter = None
        
        self.__joules_per_square_millimeter = None
        
        self.__watt_hours_per_square_meter = None
        
        self.__btus_per_square_foot = None
        
        self.__kilojoules_per_square_meter = None
        
        self.__millijoules_per_square_centimeter = None
        
        self.__kilowatt_hours_per_square_meter = None
        
        self.__kilobtus_per_square_foot = None
        

    def convert(self, unit: IrradiationUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: IrradiationUnits = IrradiationUnits.JoulePerSquareMeter) -> IrradiationDto:
        """
        Get a new instance of Irradiation DTO representing the current unit.

        :param hold_in_unit: The specific Irradiation unit to store the Irradiation value in the DTO representation.
        :type hold_in_unit: IrradiationUnits
        :return: A new instance of IrradiationDto.
        :rtype: IrradiationDto
        """
        return IrradiationDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: IrradiationUnits = IrradiationUnits.JoulePerSquareMeter):
        """
        Get a Irradiation DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Irradiation unit to store the Irradiation value in the DTO representation.
        :type hold_in_unit: IrradiationUnits
        :return: JSON object represents Irradiation DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "JoulePerSquareMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(irradiation_dto: IrradiationDto):
        """
        Obtain a new instance of Irradiation from a DTO unit object.

        :param irradiation_dto: The Irradiation DTO representation.
        :type irradiation_dto: IrradiationDto
        :return: A new instance of Irradiation.
        :rtype: Irradiation
        """
        return Irradiation(irradiation_dto.value, irradiation_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Irradiation from a DTO unit json representation.

        :param data: The Irradiation DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "JoulePerSquareMeter"}
        :return: A new instance of Irradiation.
        :rtype: Irradiation
        """
        return Irradiation.from_dto(IrradiationDto.from_json(data))

    def __convert_from_base(self, from_unit: IrradiationUnits) -> float:
        value = self._value
        
        if from_unit == IrradiationUnits.JoulePerSquareMeter:
            return (value)
        
        if from_unit == IrradiationUnits.JoulePerSquareCentimeter:
            return (value / 1e4)
        
        if from_unit == IrradiationUnits.JoulePerSquareMillimeter:
            return (value / 1e6)
        
        if from_unit == IrradiationUnits.WattHourPerSquareMeter:
            return (value / 3600)
        
        if from_unit == IrradiationUnits.BtuPerSquareFoot:
            return (value / (52752792631 / 4645152))
        
        if from_unit == IrradiationUnits.KilojoulePerSquareMeter:
            return ((value) / 1000.0)
        
        if from_unit == IrradiationUnits.MillijoulePerSquareCentimeter:
            return ((value / 1e4) / 0.001)
        
        if from_unit == IrradiationUnits.KilowattHourPerSquareMeter:
            return ((value / 3600) / 1000.0)
        
        if from_unit == IrradiationUnits.KilobtuPerSquareFoot:
            return ((value / (52752792631 / 4645152)) / 1000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: IrradiationUnits) -> float:
        
        if to_unit == IrradiationUnits.JoulePerSquareMeter:
            return (value)
        
        if to_unit == IrradiationUnits.JoulePerSquareCentimeter:
            return (value * 1e4)
        
        if to_unit == IrradiationUnits.JoulePerSquareMillimeter:
            return (value * 1e6)
        
        if to_unit == IrradiationUnits.WattHourPerSquareMeter:
            return (value * 3600)
        
        if to_unit == IrradiationUnits.BtuPerSquareFoot:
            return (value * (52752792631 / 4645152))
        
        if to_unit == IrradiationUnits.KilojoulePerSquareMeter:
            return ((value) * 1000.0)
        
        if to_unit == IrradiationUnits.MillijoulePerSquareCentimeter:
            return ((value * 1e4) * 0.001)
        
        if to_unit == IrradiationUnits.KilowattHourPerSquareMeter:
            return ((value * 3600) * 1000.0)
        
        if to_unit == IrradiationUnits.KilobtuPerSquareFoot:
            return ((value * (52752792631 / 4645152)) * 1000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_joules_per_square_meter(joules_per_square_meter: float):
        """
        Create a new instance of Irradiation from a value in joules_per_square_meter.

        

        :param meters: The Irradiation value in joules_per_square_meter.
        :type joules_per_square_meter: float
        :return: A new instance of Irradiation.
        :rtype: Irradiation
        """
        return Irradiation(joules_per_square_meter, IrradiationUnits.JoulePerSquareMeter)

    
    @staticmethod
    def from_joules_per_square_centimeter(joules_per_square_centimeter: float):
        """
        Create a new instance of Irradiation from a value in joules_per_square_centimeter.

        

        :param meters: The Irradiation value in joules_per_square_centimeter.
        :type joules_per_square_centimeter: float
        :return: A new instance of Irradiation.
        :rtype: Irradiation
        """
        return Irradiation(joules_per_square_centimeter, IrradiationUnits.JoulePerSquareCentimeter)

    
    @staticmethod
    def from_joules_per_square_millimeter(joules_per_square_millimeter: float):
        """
        Create a new instance of Irradiation from a value in joules_per_square_millimeter.

        

        :param meters: The Irradiation value in joules_per_square_millimeter.
        :type joules_per_square_millimeter: float
        :return: A new instance of Irradiation.
        :rtype: Irradiation
        """
        return Irradiation(joules_per_square_millimeter, IrradiationUnits.JoulePerSquareMillimeter)

    
    @staticmethod
    def from_watt_hours_per_square_meter(watt_hours_per_square_meter: float):
        """
        Create a new instance of Irradiation from a value in watt_hours_per_square_meter.

        

        :param meters: The Irradiation value in watt_hours_per_square_meter.
        :type watt_hours_per_square_meter: float
        :return: A new instance of Irradiation.
        :rtype: Irradiation
        """
        return Irradiation(watt_hours_per_square_meter, IrradiationUnits.WattHourPerSquareMeter)

    
    @staticmethod
    def from_btus_per_square_foot(btus_per_square_foot: float):
        """
        Create a new instance of Irradiation from a value in btus_per_square_foot.

        

        :param meters: The Irradiation value in btus_per_square_foot.
        :type btus_per_square_foot: float
        :return: A new instance of Irradiation.
        :rtype: Irradiation
        """
        return Irradiation(btus_per_square_foot, IrradiationUnits.BtuPerSquareFoot)

    
    @staticmethod
    def from_kilojoules_per_square_meter(kilojoules_per_square_meter: float):
        """
        Create a new instance of Irradiation from a value in kilojoules_per_square_meter.

        

        :param meters: The Irradiation value in kilojoules_per_square_meter.
        :type kilojoules_per_square_meter: float
        :return: A new instance of Irradiation.
        :rtype: Irradiation
        """
        return Irradiation(kilojoules_per_square_meter, IrradiationUnits.KilojoulePerSquareMeter)

    
    @staticmethod
    def from_millijoules_per_square_centimeter(millijoules_per_square_centimeter: float):
        """
        Create a new instance of Irradiation from a value in millijoules_per_square_centimeter.

        

        :param meters: The Irradiation value in millijoules_per_square_centimeter.
        :type millijoules_per_square_centimeter: float
        :return: A new instance of Irradiation.
        :rtype: Irradiation
        """
        return Irradiation(millijoules_per_square_centimeter, IrradiationUnits.MillijoulePerSquareCentimeter)

    
    @staticmethod
    def from_kilowatt_hours_per_square_meter(kilowatt_hours_per_square_meter: float):
        """
        Create a new instance of Irradiation from a value in kilowatt_hours_per_square_meter.

        

        :param meters: The Irradiation value in kilowatt_hours_per_square_meter.
        :type kilowatt_hours_per_square_meter: float
        :return: A new instance of Irradiation.
        :rtype: Irradiation
        """
        return Irradiation(kilowatt_hours_per_square_meter, IrradiationUnits.KilowattHourPerSquareMeter)

    
    @staticmethod
    def from_kilobtus_per_square_foot(kilobtus_per_square_foot: float):
        """
        Create a new instance of Irradiation from a value in kilobtus_per_square_foot.

        

        :param meters: The Irradiation value in kilobtus_per_square_foot.
        :type kilobtus_per_square_foot: float
        :return: A new instance of Irradiation.
        :rtype: Irradiation
        """
        return Irradiation(kilobtus_per_square_foot, IrradiationUnits.KilobtuPerSquareFoot)

    
    @property
    def joules_per_square_meter(self) -> float:
        """
        
        """
        if self.__joules_per_square_meter != None:
            return self.__joules_per_square_meter
        self.__joules_per_square_meter = self.__convert_from_base(IrradiationUnits.JoulePerSquareMeter)
        return self.__joules_per_square_meter

    
    @property
    def joules_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__joules_per_square_centimeter != None:
            return self.__joules_per_square_centimeter
        self.__joules_per_square_centimeter = self.__convert_from_base(IrradiationUnits.JoulePerSquareCentimeter)
        return self.__joules_per_square_centimeter

    
    @property
    def joules_per_square_millimeter(self) -> float:
        """
        
        """
        if self.__joules_per_square_millimeter != None:
            return self.__joules_per_square_millimeter
        self.__joules_per_square_millimeter = self.__convert_from_base(IrradiationUnits.JoulePerSquareMillimeter)
        return self.__joules_per_square_millimeter

    
    @property
    def watt_hours_per_square_meter(self) -> float:
        """
        
        """
        if self.__watt_hours_per_square_meter != None:
            return self.__watt_hours_per_square_meter
        self.__watt_hours_per_square_meter = self.__convert_from_base(IrradiationUnits.WattHourPerSquareMeter)
        return self.__watt_hours_per_square_meter

    
    @property
    def btus_per_square_foot(self) -> float:
        """
        
        """
        if self.__btus_per_square_foot != None:
            return self.__btus_per_square_foot
        self.__btus_per_square_foot = self.__convert_from_base(IrradiationUnits.BtuPerSquareFoot)
        return self.__btus_per_square_foot

    
    @property
    def kilojoules_per_square_meter(self) -> float:
        """
        
        """
        if self.__kilojoules_per_square_meter != None:
            return self.__kilojoules_per_square_meter
        self.__kilojoules_per_square_meter = self.__convert_from_base(IrradiationUnits.KilojoulePerSquareMeter)
        return self.__kilojoules_per_square_meter

    
    @property
    def millijoules_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__millijoules_per_square_centimeter != None:
            return self.__millijoules_per_square_centimeter
        self.__millijoules_per_square_centimeter = self.__convert_from_base(IrradiationUnits.MillijoulePerSquareCentimeter)
        return self.__millijoules_per_square_centimeter

    
    @property
    def kilowatt_hours_per_square_meter(self) -> float:
        """
        
        """
        if self.__kilowatt_hours_per_square_meter != None:
            return self.__kilowatt_hours_per_square_meter
        self.__kilowatt_hours_per_square_meter = self.__convert_from_base(IrradiationUnits.KilowattHourPerSquareMeter)
        return self.__kilowatt_hours_per_square_meter

    
    @property
    def kilobtus_per_square_foot(self) -> float:
        """
        
        """
        if self.__kilobtus_per_square_foot != None:
            return self.__kilobtus_per_square_foot
        self.__kilobtus_per_square_foot = self.__convert_from_base(IrradiationUnits.KilobtuPerSquareFoot)
        return self.__kilobtus_per_square_foot

    
    def to_string(self, unit: IrradiationUnits = IrradiationUnits.JoulePerSquareMeter, fractional_digits: int = None) -> str:
        """
        Format the Irradiation to a string.
        
        Note: the default format for Irradiation is JoulePerSquareMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Irradiation. Default is 'JoulePerSquareMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == IrradiationUnits.JoulePerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.joules_per_square_meter, fractional_digits)} J/m²"""
        
        if unit == IrradiationUnits.JoulePerSquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.joules_per_square_centimeter, fractional_digits)} J/cm²"""
        
        if unit == IrradiationUnits.JoulePerSquareMillimeter:
            return f"""{super()._truncate_fraction_digits(self.joules_per_square_millimeter, fractional_digits)} J/mm²"""
        
        if unit == IrradiationUnits.WattHourPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.watt_hours_per_square_meter, fractional_digits)} Wh/m²"""
        
        if unit == IrradiationUnits.BtuPerSquareFoot:
            return f"""{super()._truncate_fraction_digits(self.btus_per_square_foot, fractional_digits)} Btu/ft²"""
        
        if unit == IrradiationUnits.KilojoulePerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.kilojoules_per_square_meter, fractional_digits)} kJ/m²"""
        
        if unit == IrradiationUnits.MillijoulePerSquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.millijoules_per_square_centimeter, fractional_digits)} mJ/cm²"""
        
        if unit == IrradiationUnits.KilowattHourPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.kilowatt_hours_per_square_meter, fractional_digits)} kWh/m²"""
        
        if unit == IrradiationUnits.KilobtuPerSquareFoot:
            return f"""{super()._truncate_fraction_digits(self.kilobtus_per_square_foot, fractional_digits)} kBtu/ft²"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: IrradiationUnits = IrradiationUnits.JoulePerSquareMeter) -> str:
        """
        Get Irradiation unit abbreviation.
        Note! the default abbreviation for Irradiation is JoulePerSquareMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == IrradiationUnits.JoulePerSquareMeter:
            return """J/m²"""
        
        if unit_abbreviation == IrradiationUnits.JoulePerSquareCentimeter:
            return """J/cm²"""
        
        if unit_abbreviation == IrradiationUnits.JoulePerSquareMillimeter:
            return """J/mm²"""
        
        if unit_abbreviation == IrradiationUnits.WattHourPerSquareMeter:
            return """Wh/m²"""
        
        if unit_abbreviation == IrradiationUnits.BtuPerSquareFoot:
            return """Btu/ft²"""
        
        if unit_abbreviation == IrradiationUnits.KilojoulePerSquareMeter:
            return """kJ/m²"""
        
        if unit_abbreviation == IrradiationUnits.MillijoulePerSquareCentimeter:
            return """mJ/cm²"""
        
        if unit_abbreviation == IrradiationUnits.KilowattHourPerSquareMeter:
            return """kWh/m²"""
        
        if unit_abbreviation == IrradiationUnits.KilobtuPerSquareFoot:
            return """kBtu/ft²"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ThermalResistanceUnits(Enum):
        """
            ThermalResistanceUnits enumeration
        """
        
        SquareMeterKelvinPerKilowatt = 'SquareMeterKelvinPerKilowatt'
        """
            
        """
        
        SquareMeterKelvinPerWatt = 'SquareMeterKelvinPerWatt'
        """
            
        """
        
        SquareMeterDegreeCelsiusPerWatt = 'SquareMeterDegreeCelsiusPerWatt'
        """
            
        """
        
        SquareCentimeterKelvinPerWatt = 'SquareCentimeterKelvinPerWatt'
        """
            
        """
        
        SquareCentimeterHourDegreeCelsiusPerKilocalorie = 'SquareCentimeterHourDegreeCelsiusPerKilocalorie'
        """
            
        """
        
        HourSquareFeetDegreeFahrenheitPerBtu = 'HourSquareFeetDegreeFahrenheitPerBtu'
        """
            
        """
        

class ThermalResistanceDto:
    """
    A DTO representation of a ThermalResistance

    Attributes:
        value (float): The value of the ThermalResistance.
        unit (ThermalResistanceUnits): The specific unit that the ThermalResistance value is representing.
    """

    def __init__(self, value: float, unit: ThermalResistanceUnits):
        """
        Create a new DTO representation of a ThermalResistance

        Parameters:
            value (float): The value of the ThermalResistance.
            unit (ThermalResistanceUnits): The specific unit that the ThermalResistance value is representing.
        """
        self.value: float = value
        """
        The value of the ThermalResistance
        """
        self.unit: ThermalResistanceUnits = unit
        """
        The specific unit that the ThermalResistance value is representing
        """

    def to_json(self):
        """
        Get a ThermalResistance DTO JSON object representing the current unit.

        :return: JSON object represents ThermalResistance DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "SquareMeterKelvinPerKilowatt"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ThermalResistance DTO from a json representation.

        :param data: The ThermalResistance DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "SquareMeterKelvinPerKilowatt"}
        :return: A new instance of ThermalResistanceDto.
        :rtype: ThermalResistanceDto
        """
        return ThermalResistanceDto(value=data["value"], unit=ThermalResistanceUnits(data["unit"]))


class ThermalResistance(AbstractMeasure):
    """
    Heat Transfer Coefficient or Thermal conductivity - indicates a materials ability to conduct heat.

    Args:
        value (float): The value.
        from_unit (ThermalResistanceUnits): The ThermalResistance unit to create from, The default unit is SquareMeterKelvinPerKilowatt
    """
    def __init__(self, value: float, from_unit: ThermalResistanceUnits = ThermalResistanceUnits.SquareMeterKelvinPerKilowatt):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__square_meter_kelvins_per_kilowatt = None
        
        self.__square_meter_kelvins_per_watt = None
        
        self.__square_meter_degrees_celsius_per_watt = None
        
        self.__square_centimeter_kelvins_per_watt = None
        
        self.__square_centimeter_hour_degrees_celsius_per_kilocalorie = None
        
        self.__hour_square_feet_degrees_fahrenheit_per_btu = None
        

    def convert(self, unit: ThermalResistanceUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ThermalResistanceUnits = ThermalResistanceUnits.SquareMeterKelvinPerKilowatt) -> ThermalResistanceDto:
        """
        Get a new instance of ThermalResistance DTO representing the current unit.

        :param hold_in_unit: The specific ThermalResistance unit to store the ThermalResistance value in the DTO representation.
        :type hold_in_unit: ThermalResistanceUnits
        :return: A new instance of ThermalResistanceDto.
        :rtype: ThermalResistanceDto
        """
        return ThermalResistanceDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ThermalResistanceUnits = ThermalResistanceUnits.SquareMeterKelvinPerKilowatt):
        """
        Get a ThermalResistance DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ThermalResistance unit to store the ThermalResistance value in the DTO representation.
        :type hold_in_unit: ThermalResistanceUnits
        :return: JSON object represents ThermalResistance DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "SquareMeterKelvinPerKilowatt"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(thermal_resistance_dto: ThermalResistanceDto):
        """
        Obtain a new instance of ThermalResistance from a DTO unit object.

        :param thermal_resistance_dto: The ThermalResistance DTO representation.
        :type thermal_resistance_dto: ThermalResistanceDto
        :return: A new instance of ThermalResistance.
        :rtype: ThermalResistance
        """
        return ThermalResistance(thermal_resistance_dto.value, thermal_resistance_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ThermalResistance from a DTO unit json representation.

        :param data: The ThermalResistance DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "SquareMeterKelvinPerKilowatt"}
        :return: A new instance of ThermalResistance.
        :rtype: ThermalResistance
        """
        return ThermalResistance.from_dto(ThermalResistanceDto.from_json(data))

    def __convert_from_base(self, from_unit: ThermalResistanceUnits) -> float:
        value = self._value
        
        if from_unit == ThermalResistanceUnits.SquareMeterKelvinPerKilowatt:
            return (value)
        
        if from_unit == ThermalResistanceUnits.SquareMeterKelvinPerWatt:
            return (value / 1000)
        
        if from_unit == ThermalResistanceUnits.SquareMeterDegreeCelsiusPerWatt:
            return (value / 1000.0)
        
        if from_unit == ThermalResistanceUnits.SquareCentimeterKelvinPerWatt:
            return (value / 0.1)
        
        if from_unit == ThermalResistanceUnits.SquareCentimeterHourDegreeCelsiusPerKilocalorie:
            return (value / 0.0859779507590433)
        
        if from_unit == ThermalResistanceUnits.HourSquareFeetDegreeFahrenheitPerBtu:
            return (value / 176.1121482159839)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ThermalResistanceUnits) -> float:
        
        if to_unit == ThermalResistanceUnits.SquareMeterKelvinPerKilowatt:
            return (value)
        
        if to_unit == ThermalResistanceUnits.SquareMeterKelvinPerWatt:
            return (value * 1000)
        
        if to_unit == ThermalResistanceUnits.SquareMeterDegreeCelsiusPerWatt:
            return (value * 1000.0)
        
        if to_unit == ThermalResistanceUnits.SquareCentimeterKelvinPerWatt:
            return (value * 0.1)
        
        if to_unit == ThermalResistanceUnits.SquareCentimeterHourDegreeCelsiusPerKilocalorie:
            return (value * 0.0859779507590433)
        
        if to_unit == ThermalResistanceUnits.HourSquareFeetDegreeFahrenheitPerBtu:
            return (value * 176.1121482159839)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_square_meter_kelvins_per_kilowatt(square_meter_kelvins_per_kilowatt: float):
        """
        Create a new instance of ThermalResistance from a value in square_meter_kelvins_per_kilowatt.

        

        :param meters: The ThermalResistance value in square_meter_kelvins_per_kilowatt.
        :type square_meter_kelvins_per_kilowatt: float
        :return: A new instance of ThermalResistance.
        :rtype: ThermalResistance
        """
        return ThermalResistance(square_meter_kelvins_per_kilowatt, ThermalResistanceUnits.SquareMeterKelvinPerKilowatt)

    
    @staticmethod
    def from_square_meter_kelvins_per_watt(square_meter_kelvins_per_watt: float):
        """
        Create a new instance of ThermalResistance from a value in square_meter_kelvins_per_watt.

        

        :param meters: The ThermalResistance value in square_meter_kelvins_per_watt.
        :type square_meter_kelvins_per_watt: float
        :return: A new instance of ThermalResistance.
        :rtype: ThermalResistance
        """
        return ThermalResistance(square_meter_kelvins_per_watt, ThermalResistanceUnits.SquareMeterKelvinPerWatt)

    
    @staticmethod
    def from_square_meter_degrees_celsius_per_watt(square_meter_degrees_celsius_per_watt: float):
        """
        Create a new instance of ThermalResistance from a value in square_meter_degrees_celsius_per_watt.

        

        :param meters: The ThermalResistance value in square_meter_degrees_celsius_per_watt.
        :type square_meter_degrees_celsius_per_watt: float
        :return: A new instance of ThermalResistance.
        :rtype: ThermalResistance
        """
        return ThermalResistance(square_meter_degrees_celsius_per_watt, ThermalResistanceUnits.SquareMeterDegreeCelsiusPerWatt)

    
    @staticmethod
    def from_square_centimeter_kelvins_per_watt(square_centimeter_kelvins_per_watt: float):
        """
        Create a new instance of ThermalResistance from a value in square_centimeter_kelvins_per_watt.

        

        :param meters: The ThermalResistance value in square_centimeter_kelvins_per_watt.
        :type square_centimeter_kelvins_per_watt: float
        :return: A new instance of ThermalResistance.
        :rtype: ThermalResistance
        """
        return ThermalResistance(square_centimeter_kelvins_per_watt, ThermalResistanceUnits.SquareCentimeterKelvinPerWatt)

    
    @staticmethod
    def from_square_centimeter_hour_degrees_celsius_per_kilocalorie(square_centimeter_hour_degrees_celsius_per_kilocalorie: float):
        """
        Create a new instance of ThermalResistance from a value in square_centimeter_hour_degrees_celsius_per_kilocalorie.

        

        :param meters: The ThermalResistance value in square_centimeter_hour_degrees_celsius_per_kilocalorie.
        :type square_centimeter_hour_degrees_celsius_per_kilocalorie: float
        :return: A new instance of ThermalResistance.
        :rtype: ThermalResistance
        """
        return ThermalResistance(square_centimeter_hour_degrees_celsius_per_kilocalorie, ThermalResistanceUnits.SquareCentimeterHourDegreeCelsiusPerKilocalorie)

    
    @staticmethod
    def from_hour_square_feet_degrees_fahrenheit_per_btu(hour_square_feet_degrees_fahrenheit_per_btu: float):
        """
        Create a new instance of ThermalResistance from a value in hour_square_feet_degrees_fahrenheit_per_btu.

        

        :param meters: The ThermalResistance value in hour_square_feet_degrees_fahrenheit_per_btu.
        :type hour_square_feet_degrees_fahrenheit_per_btu: float
        :return: A new instance of ThermalResistance.
        :rtype: ThermalResistance
        """
        return ThermalResistance(hour_square_feet_degrees_fahrenheit_per_btu, ThermalResistanceUnits.HourSquareFeetDegreeFahrenheitPerBtu)

    
    @property
    def square_meter_kelvins_per_kilowatt(self) -> float:
        """
        
        """
        if self.__square_meter_kelvins_per_kilowatt != None:
            return self.__square_meter_kelvins_per_kilowatt
        self.__square_meter_kelvins_per_kilowatt = self.__convert_from_base(ThermalResistanceUnits.SquareMeterKelvinPerKilowatt)
        return self.__square_meter_kelvins_per_kilowatt

    
    @property
    def square_meter_kelvins_per_watt(self) -> float:
        """
        
        """
        if self.__square_meter_kelvins_per_watt != None:
            return self.__square_meter_kelvins_per_watt
        self.__square_meter_kelvins_per_watt = self.__convert_from_base(ThermalResistanceUnits.SquareMeterKelvinPerWatt)
        return self.__square_meter_kelvins_per_watt

    
    @property
    def square_meter_degrees_celsius_per_watt(self) -> float:
        """
        
        """
        if self.__square_meter_degrees_celsius_per_watt != None:
            return self.__square_meter_degrees_celsius_per_watt
        self.__square_meter_degrees_celsius_per_watt = self.__convert_from_base(ThermalResistanceUnits.SquareMeterDegreeCelsiusPerWatt)
        return self.__square_meter_degrees_celsius_per_watt

    
    @property
    def square_centimeter_kelvins_per_watt(self) -> float:
        """
        
        """
        if self.__square_centimeter_kelvins_per_watt != None:
            return self.__square_centimeter_kelvins_per_watt
        self.__square_centimeter_kelvins_per_watt = self.__convert_from_base(ThermalResistanceUnits.SquareCentimeterKelvinPerWatt)
        return self.__square_centimeter_kelvins_per_watt

    
    @property
    def square_centimeter_hour_degrees_celsius_per_kilocalorie(self) -> float:
        """
        
        """
        if self.__square_centimeter_hour_degrees_celsius_per_kilocalorie != None:
            return self.__square_centimeter_hour_degrees_celsius_per_kilocalorie
        self.__square_centimeter_hour_degrees_celsius_per_kilocalorie = self.__convert_from_base(ThermalResistanceUnits.SquareCentimeterHourDegreeCelsiusPerKilocalorie)
        return self.__square_centimeter_hour_degrees_celsius_per_kilocalorie

    
    @property
    def hour_square_feet_degrees_fahrenheit_per_btu(self) -> float:
        """
        
        """
        if self.__hour_square_feet_degrees_fahrenheit_per_btu != None:
            return self.__hour_square_feet_degrees_fahrenheit_per_btu
        self.__hour_square_feet_degrees_fahrenheit_per_btu = self.__convert_from_base(ThermalResistanceUnits.HourSquareFeetDegreeFahrenheitPerBtu)
        return self.__hour_square_feet_degrees_fahrenheit_per_btu

    
    def to_string(self, unit: ThermalResistanceUnits = ThermalResistanceUnits.SquareMeterKelvinPerKilowatt, fractional_digits: int = None) -> str:
        """
        Format the ThermalResistance to a string.
        
        Note: the default format for ThermalResistance is SquareMeterKelvinPerKilowatt.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ThermalResistance. Default is 'SquareMeterKelvinPerKilowatt'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ThermalResistanceUnits.SquareMeterKelvinPerKilowatt:
            return f"""{super()._truncate_fraction_digits(self.square_meter_kelvins_per_kilowatt, fractional_digits)} m²K/kW"""
        
        if unit == ThermalResistanceUnits.SquareMeterKelvinPerWatt:
            return f"""{super()._truncate_fraction_digits(self.square_meter_kelvins_per_watt, fractional_digits)} m²K/W"""
        
        if unit == ThermalResistanceUnits.SquareMeterDegreeCelsiusPerWatt:
            return f"""{super()._truncate_fraction_digits(self.square_meter_degrees_celsius_per_watt, fractional_digits)} m²°C/W"""
        
        if unit == ThermalResistanceUnits.SquareCentimeterKelvinPerWatt:
            return f"""{super()._truncate_fraction_digits(self.square_centimeter_kelvins_per_watt, fractional_digits)} cm²K/W"""
        
        if unit == ThermalResistanceUnits.SquareCentimeterHourDegreeCelsiusPerKilocalorie:
            return f"""{super()._truncate_fraction_digits(self.square_centimeter_hour_degrees_celsius_per_kilocalorie, fractional_digits)} cm²Hr°C/kcal"""
        
        if unit == ThermalResistanceUnits.HourSquareFeetDegreeFahrenheitPerBtu:
            return f"""{super()._truncate_fraction_digits(self.hour_square_feet_degrees_fahrenheit_per_btu, fractional_digits)} Hrft²°F/Btu"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ThermalResistanceUnits = ThermalResistanceUnits.SquareMeterKelvinPerKilowatt) -> str:
        """
        Get ThermalResistance unit abbreviation.
        Note! the default abbreviation for ThermalResistance is SquareMeterKelvinPerKilowatt.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ThermalResistanceUnits.SquareMeterKelvinPerKilowatt:
            return """m²K/kW"""
        
        if unit_abbreviation == ThermalResistanceUnits.SquareMeterKelvinPerWatt:
            return """m²K/W"""
        
        if unit_abbreviation == ThermalResistanceUnits.SquareMeterDegreeCelsiusPerWatt:
            return """m²°C/W"""
        
        if unit_abbreviation == ThermalResistanceUnits.SquareCentimeterKelvinPerWatt:
            return """cm²K/W"""
        
        if unit_abbreviation == ThermalResistanceUnits.SquareCentimeterHourDegreeCelsiusPerKilocalorie:
            return """cm²Hr°C/kcal"""
        
        if unit_abbreviation == ThermalResistanceUnits.HourSquareFeetDegreeFahrenheitPerBtu:
            return """Hrft²°F/Btu"""
        